@@ -40,22 +40,11 @@ ASObject::ASObject(QVariantMap objectMap,
     // FIXME: kind of basic...
     location = objectMap.value("location").toMap().value("displayName").toString();
 
-    authorMap = objectMap.value("author").toMap();
 
-    // FIXME: this "author code" is duplicated from ASActivity and other places
-    authorName = authorMap.value("displayName").toString();
-    authorId = authorMap.value("id").toString();
-    // For Author ID, remove the first 5 characters from the field, if they are "acct:"
-    if (authorId.startsWith("acct:"))
-    {
-        authorId.remove(0,5);
-    }
-    authorAvatar = authorMap.value("image").toMap().value("url").toString();
-    authorUrl = authorMap.value("url").toString();
-    authorHometown = authorMap.value("location").toMap().value("displayName").toString();
-    authorBio = authorMap.value("summary").toString();
+    // Author
+    this->asAuthor = new ASPerson(objectMap.value("author").toMap(), this);
 
-    // This will hold the date when the post was deleted, or empty if it wasn't
+    // This will hold the date when the object was deleted, or empty if it wasn't
     deleted = objectMap.value("deleted").toString();
 
     liked = objectMap.value("liked").toString();
@@ -63,6 +52,7 @@ ASObject::ASObject(QVariantMap objectMap,
     // The object to which this one replies, if any (note to which a comment replies, etc)
     inReplyToMap = objectMap.value("inReplyTo").toMap();
 
+
     ///
     /// End of "meta"; Start of content
     ///
@@ -98,6 +88,42 @@ ASObject::ASObject(QVariantMap objectMap,
     }
 
 
+    // Get audio file URL
+    if (type == "audio")
+    {
+        audioUrl = objectMap.value("stream").toMap().value("pump_io").toMap().value("proxyURL").toString();
+        if (audioUrl.isEmpty())
+        {
+            qDebug() << "No proxyed link to audio, fetching regular link";
+            audioUrl = objectMap.value("stream").toMap().value("url").toString();
+        }
+    }
+
+    // Get video file URL
+    if (type == "video")
+    {
+        videoUrl = objectMap.value("stream").toMap().value("pump_io").toMap().value("proxyURL").toString();
+        if (videoUrl.isEmpty())
+        {
+            qDebug() << "No proxyed link to video, fetching regular link";
+            videoUrl = objectMap.value("stream").toMap().value("url").toString();
+        }
+    }
+
+    // Get general file URL
+    if (type == "file")
+    {
+        fileUrl = objectMap.value("fileUrl").toMap().value("pump_io").toMap().value("proxyURL").toString();
+        if (fileUrl.isEmpty())
+        {
+            qDebug() << "No proxyed link to general file, fetching regular link";
+            fileUrl = objectMap.value("fileUrl").toString();
+        }
+        // FIXME: get mimeType
+        qDebug() << "file mimeType:" << objectMap.value("mimeType").toString();
+    }
+
+
     // Title can be in non-image posts, too!
     title = objectMap.value("displayName").toString();
     content = objectMap.value("content").toString();
@@ -137,7 +163,13 @@ ASObject::~ASObject()
 }
 
 
-// Getters
+
+/////// Getters
+
+ASPerson *ASObject::author()
+{
+    return this->asAuthor;
+}
 
 QString ASObject::getId()
 {
@@ -180,40 +212,6 @@ QString ASObject::isLiked()
     return this->liked;
 }
 
-QVariantMap ASObject::getAuthorMap()
-{
-    return this->authorMap;
-}
-
-QString ASObject::getAuthorName()
-{
-    return this->authorName;
-}
-
-QString ASObject::getAuthorId()
-{
-    return this->authorId;
-}
-
-QString ASObject::getAuthorAvatar()
-{
-    return this->authorAvatar;
-}
-
-QString ASObject::getAuthorUrl()
-{
-    return this->authorUrl;
-}
-
-QString ASObject::getAuthorHometown()
-{
-    return this->authorHometown;
-}
-
-QString ASObject::getAuthorBio()
-{
-    return this->authorBio;
-}
 
 QString ASObject::getTitle()
 {
@@ -230,6 +228,21 @@ QString ASObject::getImageUrl()
     return this->imageUrl;
 }
 
+QString ASObject::getAudioUrl()
+{
+    return this->audioUrl;
+}
+
+QString ASObject::getVideoUrl()
+{
+    return this->videoUrl;
+}
+
+QString ASObject::getFileUrl()
+{
+    return this->fileUrl;
+}
+
 QString ASObject::getLikesCount()
 {
     return this->likesCount;
