@@ -26,6 +26,7 @@
 
 #include <QDebug>
 
+#include "asperson.h"
 
 class ASObject : public QObject
 {
@@ -36,6 +37,8 @@ public:
                       QObject *parent = 0);
     ~ASObject();
 
+    ASPerson *author();
+
     QString getId();
     QString getType();
     QString getUrl();
@@ -46,18 +49,13 @@ public:
     QString getDeletedTime();
     QString isLiked();
 
-    QVariantMap getAuthorMap();
-    QString getAuthorName();
-    QString getAuthorId();
-    QString getAuthorAvatar();
-    QString getAuthorUrl();
-    QString getAuthorHometown();
-    QString getAuthorBio();
-
     QString getTitle();
     QString getContent();
 
     QString getImageUrl();
+    QString getAudioUrl();
+    QString getVideoUrl();
+    QString getFileUrl();
 
     QString getLikesCount();
     QString getCommentsCount();
@@ -79,6 +77,8 @@ signals:
 public slots:
 
 private:
+    ASPerson *asAuthor;
+
     QString id;
     QString type;
     QString url;
@@ -90,18 +90,13 @@ private:
     QString liked;
 
 
-    QVariantMap authorMap;
-    QString authorName;
-    QString authorId;
-    QString authorAvatar;
-    QString authorUrl;
-    QString authorHometown;
-    QString authorBio;
-
     QString title;
     QString content;
 
     QString imageUrl;
+    QString audioUrl;
+    QString videoUrl;
+    QString fileUrl;
 
     QString likesCount;
     QString commentsCount;
