@@ -31,6 +31,7 @@
 #include <QLineEdit>
 #include <QPushButton>
 #include <QKeyEvent>
+#include <QCloseEvent>
 
 #include <QDebug>
 
@@ -53,6 +54,7 @@ public:
 
 protected:
     virtual void keyPressEvent(QKeyEvent *event);
+    virtual void closeEvent(QCloseEvent *event);
 
 signals:
     void audienceSelected(QString selectorType,
@@ -60,7 +62,8 @@ signals:
 
 
 public slots:
-    void copyToSelected(QString contactString);
+    void copyToSelected(QIcon contactIcon,
+                        QString contactString);
 
     void setAudience();
 
