@@ -31,6 +31,7 @@
 
 #include "pumpcontroller.h"
 #include "composer.h"
+#include "asobject.h"
 #include "comment.h"
 
 
@@ -43,10 +44,14 @@ public:
     ~CommenterBlock();
 
     void clearComments();
-    void setComments(QVariantList commentsList);
+    void setComments(QVariantList commentsList, int commentCount);
+    void updateFuzzyTimestamps();
+
+    void adjustCommentsWidth();
 
 signals:
-    void commentSent(QString comment);
+    void commentSent(QString commentText);
+    void commentUpdated(QString commentId, QString commentText);
     void allCommentsRequested();
 
 
@@ -56,6 +61,8 @@ public slots:
 
     void quoteComment(QString content);
 
+    void editComment(QString id, QString content);
+
     void requestAllComments();
     void onPostingCommentOk();
     void onPostingCommentFailed();
@@ -85,6 +92,9 @@ private:
     QPushButton *commentButton;
     QPushButton *cancelButton;
 
+    bool editingMode;
+    QString editingCommentId;
+
 
     QList<Comment *> commentsInBlock;
 
