@@ -93,7 +93,8 @@ Composer::Composer(bool forPublisher)
     toolsMenu->addAction(QIcon::fromTheme("format-text-italic"),
                          tr("Quote block"),
                          this,
-                         SLOT(makeQuote()));
+                         SLOT(makeQuote()),
+                         QKeySequence("Ctrl+O"));
 
     toolsMenu->addSeparator();
 
@@ -122,7 +123,6 @@ Composer::Composer(bool forPublisher)
                                      "Button for text formatting and related options"));
     toolsButton->setMenu(toolsMenu);
     toolsButton->setToolTip(tr("Text Formatting Options"));
-    toolsButton->setHidden(true); // Hidden on startup (Publisher in miniMode)
 
 
     // Extra action for the context menu, paste as plaintext
@@ -170,8 +170,6 @@ void Composer::erase()
     if (this->forPublisher)
     {
         startConversationLabel->show();
-
-        toolsButton->hide();  /// FIXME: maybe move this line to focusOutEvent
     }
 }
 
@@ -207,11 +205,9 @@ void Composer::focusInEvent(QFocusEvent *event)
     {
         startConversationLabel->hide(); // hide placeholder message        
     }
-    toolsButton->show();
 
     QTextEdit::focusInEvent(event); // process standard event: allows context menu
 
-
     qDebug() << "Composer box got focus";
 }
 
@@ -435,7 +431,7 @@ void Composer::makeQuote()
     {
         this->textCursor().removeSelectedText();
         this->insertHtml("<br />"
-                         "<blockquote>&ldquo;" + selectedText + "&rdquo;</blockquote> ");
+                         "<blockquote>&ldquo;" + selectedText + "&rdquo;</blockquote><br>");
     }
     // FIXME: Qt's HTML changes <blockquote> into its own formatting
     // so other clients and the web UI might not display it as other people's blockquote's
@@ -460,7 +456,10 @@ void Composer::makeLink()
     {
         link = QInputDialog::getText(this,
                                      tr("Insert a link"),
-                                     tr("Type or paste a web address here.\n"),
+                                     tr("Type or paste a web address here.\n"
+                                        "You could also select some text first, "
+                                        "to turn it into a link.")
+                                     + "\n\n",
                                      QLineEdit::Normal,
                                      "http://");
     }
@@ -470,7 +469,8 @@ void Composer::makeLink()
                                      tr("Make a link from selected text"),
                                      tr("Type or paste a web address here.\n"
                                         "The selected text (%1) will be converted "
-                                        "to a link.").arg(selectedText),
+                                        "to a link.").arg(selectedText)
+                                     + "\n\n",
                                      QLineEdit::Normal,
                                      "http://");
     }
@@ -504,7 +504,9 @@ void Composer::insertImage()
     QString imageURL;
     imageURL = QInputDialog::getText(this,
                                      tr("Insert an image from a URL"),
-                                     tr("Type or paste the image address here.\n"),
+                                     tr("Type or paste the image address here.\n"
+                                        "The link must point to the image file directly.")
+                                     + "\n\n",
                                      QLineEdit::Normal,
                                      "http://");
 
