@@ -26,13 +26,20 @@
 #include <QFormLayout>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
+#include <QTabWidget>
 #include <QLabel>
 #include <QSpinBox>
 #include <QComboBox>
 #include <QCheckBox>
 #include <QPushButton>
+#include <QSettings>
+#include <QAction>
+
 #include <QDebug>
 
+#include "colorpicker.h"
+#include "notifications.h"
+
 
 class ConfigDialog : public QWidget
 {
@@ -46,19 +53,14 @@ public:
                  int tabsPosition,
                  bool tabsMovable,
                  bool publicPosts,
-                 int showNotifications,
+                 FDNotifications *notifier,
                  QWidget *parent);
     ~ConfigDialog();
 
+    void syncNotifierOptions();
 
 signals:
-    void configurationChanged(int newUpdateInterval,
-                              int newPostsPerPageMain,
-                              int newPostsPerPageOther,
-                              int newTabsPosition,
-                              bool newTabsMovable,
-                              bool newPublicPosts,
-                              int newShowNotifications);
+    void configurationChanged();
 
 
 public slots:
@@ -68,7 +70,11 @@ public slots:
 private:
     QVBoxLayout *mainLayout;
 
-    QFormLayout *optionsLayout;
+    QTabWidget *tabWidget;
+
+    // Tab 1, general options
+    QWidget *generalOptionsWidget;
+    QFormLayout *generalOptionsLayout;
 
     QSpinBox *updateIntervalSpinbox;
     QSpinBox *postsPerPageMainSpinbox;
@@ -76,13 +82,41 @@ private:
     QComboBox *tabsPositionCombobox;
     QCheckBox *tabsMovableCheckbox;
     QCheckBox *publicPostsCheckbox;
-    QComboBox *showNotificationsCombobox;
 
+
+    // Tab 2, color options
+    QWidget *colorOptionsWidget;
+    QVBoxLayout *colorOptionsLayout;
+
+    ColorPicker *colorPicker1;
+    ColorPicker *colorPicker2;
+    ColorPicker *colorPicker3;
+    ColorPicker *colorPicker4;
+    ColorPicker *colorPicker5;
+
+
+    // Tab 3, notification options
+    QWidget *notificationOptionsWidget;
+    QFormLayout *notificationOptionsLayout;
+
+    QComboBox *notificationStyleCombobox;
+    QCheckBox *notifyNewTLCheckbox;
+    QCheckBox *notifyHLTLCheckbox;
+    QCheckBox *notifyNewMWCheckbox;
+    QCheckBox *notifyHLMWCheckbox;
+
+
+
+    // Widgets below the tab widget
     QLabel *dataDirectoryLabel;
 
     QHBoxLayout *buttonsLayout;
     QPushButton *saveConfigButton;
     QPushButton *cancelButton;
+
+    QAction *closeAction;
+
+    FDNotifications *fdNotifier;
 };
 
 #endif // CONFIGDIALOG_H
