@@ -21,116 +21,138 @@
 #include "contactcard.h"
 
 ContactCard::ContactCard(PumpController *pumpController,
-                         QMap<QString, QString> contactData,
+                         ASPerson *asPerson,
                          QWidget *parent)  :  QFrame(parent)
 {
     this->pController = pumpController;
+    asPerson->setParent(this); // reparent the person object
+
 
     this->setFrameStyle(QFrame::Box | QFrame::Raised);
 
-    this->setSizePolicy(QSizePolicy::MinimumExpanding,
-                        QSizePolicy::Maximum);
+    this->setSizePolicy(QSizePolicy::Minimum,
+                        QSizePolicy::MinimumExpanding);
 
-    mainLayout = new QHBoxLayout();
+
+    // Left widget, user avatar
     avatarLabel = new QLabel();
 
-    this->contactAvatarUrl = contactData.value("avatar");
+    this->contactAvatarUrl = asPerson->getAvatar();
 
     // Get local file name, which is stored in base64 hash form
     QString avatarFilename = MiscHelpers::getCachedAvatarFilename(contactAvatarUrl);
 
+    bool validAvatar = false;
     QFile avatarFile(avatarFilename);
     if (avatarFile.exists(avatarFilename) && !contactAvatarUrl.isEmpty())
     {
         // Load avatar if already cached
-        avatarLabel->setPixmap(QPixmap(avatarFilename).scaled(48, 48,
+        QPixmap avatarPixmap = QPixmap(avatarFilename).scaled(64, 64,
                                                               Qt::KeepAspectRatio,
-                                                              Qt::SmoothTransformation));
-        qDebug() << "ContactCard: Using cached avatar";
+                                                              Qt::SmoothTransformation);
+        if (!avatarPixmap.isNull())
+        {
+            avatarLabel->setPixmap(avatarPixmap);
+            validAvatar = true;
+            qDebug() << "ContactCard: Using cached avatar";
+        }
     }
-    else
+
+    if (!validAvatar)
     {
         // Placeholder image
-        avatarLabel->setPixmap(QIcon::fromTheme("user-identity").pixmap(48,48));
+        avatarLabel->setPixmap(QIcon::fromTheme("user-identity",
+                                                QIcon(":/images/no-avatar.png")).pixmap(64, 64));
         qDebug() << "ContactCard: Using placeholder, downloading avatar";
 
-        connect(pController, SIGNAL(avatarStored(QString,QString)),
-                this, SLOT(redrawAvatar(QString,QString)));
+        if (!contactAvatarUrl.isEmpty())
+        {
+            connect(pController, SIGNAL(avatarStored(QString,QString)),
+                    this, SLOT(redrawAvatar(QString,QString)));
 
-        // Download avatar for next time (if empty, pumpController will do nothing)
-        pController->getAvatar(contactAvatarUrl);
+            // Download avatar for next time
+            pController->enqueueAvatarForDownload(contactAvatarUrl);
+        }
     }
 
 
-    centerLayout = new QVBoxLayout();
-
+    // Center widget, user info
     QFont nameFont;
     nameFont.setBold(true);
     nameFont.setUnderline(true);
 
-    QString contactNameString = contactData.value("name");
+    this->contactName = asPerson->getName();
+    this->contactId = asPerson->getId();
+    this->contactUrl = asPerson->getUrl();
 
-    nameLabel = new QLabel();
-    nameLabel->setText(contactNameString);
-    nameLabel->setFont(nameFont);
-
-
-    this->contactID = contactData.value("id");
-    this->contactURL = contactData.value("url");
+    QString userInfoString = QString("<b><u>%1</u></b><br>").arg(contactName);
+    userInfoString.append(QString("%1<br>").arg(contactId));
+    userInfoString.append("<small>"
+                          + tr("Hometown")
+                          + QString(": %1").arg(asPerson->getHometown())
+                          + "</small>");
+    QString userCreationDate = asPerson->getCreatedAt();
+    if (!userCreationDate.isEmpty())
+    {
+        userInfoString.append("<br><small>"
+                              + tr("Joined: %1")
+                                .arg(Timestamp::fuzzyTime(userCreationDate))
+                              + "</small>");
+    }
+    else
+    {
+        userCreationDate = asPerson->getupdatedAt();
+        userInfoString.append("<br><small>"
+                              + tr("Updated: %1")
+                                .arg(Timestamp::fuzzyTime(userCreationDate))
+                              + "</small>");
+    }
 
     userInfoLabel = new QLabel();
     userInfoLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
-    userInfoLabel->setText(QString("<%1>\n").arg(contactID)
-                      + tr("Hometown") + QString(": %2").arg(contactData.value("hometown")));
+    userInfoLabel->setText(userInfoString);
     userInfoLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     userInfoLabel->setWordWrap(true);
 
 
     // Bio as tooltip for the whole contact card
-    QString contactBio = contactData.value("bio");
+    QString contactBio = asPerson->getBio().replace("\n", "<br>");
     if (!contactBio.isEmpty())
     {
-        // Make it rich text, so that it gets wordwrap
-        this->setToolTip("<b><u>"
+        this->setToolTip("<b><u>" // Make it rich text, so that it gets wordwrapped
                          + tr("Bio for %1",
                               "Abbreviation for Biography, "
                               "but you can use the full word; "
                               "%1=contact name")
-                           .arg(contactNameString)
+                           .arg(contactName)
                          + "</u></b>"
                          "<br><br>"
                          + contactBio);
     }
     else
     {
-        if (contactNameString.isEmpty())
+        if (contactName.isEmpty())
         {
             this->setToolTip(tr("This user doesn't have a biography"));
         }
         else
         {
             this->setToolTip(tr("No biography for %1",
-                                "%1=contact name").arg(contactNameString));
+                                "%1=contact name").arg(contactName));
         }
     }
 
 
-    centerLayout->addWidget(nameLabel);
-    centerLayout->addWidget(userInfoLabel);
-
-
-
-    rightLayout = new QVBoxLayout();
-
+    // Right column, buttons
     followButton = new QPushButton("*follow*");
     followButton->setFlat(true);    
-    if (contactData.value("following") == "false")
+    if (asPerson->isFollowed())
     {
-        this->setButtonToFollow();
+        this->setButtonToUnfollow();
     }
     else
     {
-        this->setButtonToUnfollow();
+        this->setButtonToFollow();
     }
 
 
@@ -148,7 +170,7 @@ ContactCard::ContactCard(PumpController *pumpController,
 
     optionsMenu = new QMenu("*options*");
     optionsMenu->addAction(openProfileAction);
-    //optionsMenu->addMenu(addToListMenu); // Don't include it for now, until 1.2 FIXME
+    //optionsMenu->addMenu(addToListMenu); // Don't include it for now, until 1.3 /FIXME
 
 
     optionsButton = new QPushButton(QIcon::fromTheme("user-properties"),
@@ -157,30 +179,33 @@ ContactCard::ContactCard(PumpController *pumpController,
     optionsButton->setMenu(optionsMenu);
 
 
+    // Layout
+    rightLayout = new QVBoxLayout();
+    rightLayout->setAlignment(Qt::AlignTop);
     rightLayout->addWidget(followButton);
     rightLayout->addWidget(optionsButton);
 
-    mainLayout->addWidget(avatarLabel,  1);  // Stretch 1/8
-    mainLayout->addLayout(centerLayout, 6);  // Stretch 6/8
-    mainLayout->addLayout(rightLayout,  1);  // 1/8
-
-
+    mainLayout = new QHBoxLayout();
+    mainLayout->addWidget(avatarLabel,   0, Qt::AlignTop);
+    mainLayout->addWidget(userInfoLabel, 1, Qt::AlignLeft);
+    mainLayout->addLayout(rightLayout);
     this->setLayout(mainLayout);
 
-    qDebug() << "ContactCard created" << this->contactID;
+    qDebug() << "ContactCard created" << this->contactId;
 }
 
 
 ContactCard::~ContactCard()
 {
-    qDebug() << "ContactCard destroyed" << this->contactID;
+    qDebug() << "ContactCard destroyed" << this->contactId;
 }
 
 
 
 void ContactCard::setButtonToFollow()
 {
-    followButton->setIcon(QIcon::fromTheme("list-add"));
+    followButton->setIcon(QIcon::fromTheme("list-add",
+                                           QIcon(":/images/list-add.png")));
     followButton->setText(tr("Follow"));
     connect(followButton, SIGNAL(clicked()),
             this, SLOT(followContact()));
@@ -191,7 +216,8 @@ void ContactCard::setButtonToFollow()
 
 void ContactCard::setButtonToUnfollow()
 {
-    followButton->setIcon(QIcon::fromTheme("list-remove"));
+    followButton->setIcon(QIcon::fromTheme("list-remove",
+                                           QIcon(":/images/list-remove.png")));
     followButton->setText(tr("Stop Following"));
     connect(followButton, SIGNAL(clicked()),
             this, SLOT(unfollowContact()));
@@ -200,6 +226,17 @@ void ContactCard::setButtonToUnfollow()
 }
 
 
+QString ContactCard::getNameAndIdString()
+{
+    return QString("%1 <%2>").arg(contactName).arg(contactId);
+}
+
+QString ContactCard::getId()
+{
+    return contactId;
+}
+
+
 
 /**************************************************************************/
 /******************************** SLOTS ***********************************/
@@ -209,7 +246,7 @@ void ContactCard::setButtonToUnfollow()
 
 void ContactCard::followContact()
 {
-    this->pController->followContact(this->contactID);
+    this->pController->followContact(this->contactId);
     this->setButtonToUnfollow();
 }
 
@@ -218,12 +255,12 @@ void ContactCard::unfollowContact()
 {
     int confirmation = QMessageBox::question(this, tr("Stop following?"),
                                              tr("Are you sure you want to stop following %1?")
-                                             .arg(this->contactID),
+                                             .arg(this->contactId),
                                              tr("&Yes, stop following"), tr("&No"), "", 1, 1);
 
     if (confirmation == 0)
     {
-        this->pController->unfollowContact(this->contactID);
+        this->pController->unfollowContact(this->contactId);
         this->setButtonToFollow();
     }
 }
@@ -231,7 +268,7 @@ void ContactCard::unfollowContact()
 
 void ContactCard::openProfileInBrowser()
 {
-    QDesktopServices::openUrl(this->contactURL);
+    QDesktopServices::openUrl(this->contactUrl);
 }
 
 
