@@ -20,38 +20,53 @@
 
 #include "filtereditor.h"
 
-FilterEditor::FilterEditor(PumpController *pumpController,
+FilterEditor::FilterEditor(FilterChecker *filterChecker,
                            QWidget *parent) : QWidget(parent)
 {
-    this->pController = pumpController;
+    this->fChecker = filterChecker;
 
     this->setWindowTitle(tr("Filter Editor") + " - Dianara");
     this->setWindowIcon(QIcon::fromTheme("view-filter"));
     this->setWindowFlags(Qt::Window);
-    this->setMinimumSize(300, 400);
+    this->setMinimumSize(520, 400);
+
+
+    ruleString = tr("%1 if %2 contains: %3",
+                    "This explains a filter rule, like: "
+                    "Hide if Author ID contains JohnDoe");
 
     QFont explanationFont;
     explanationFont.setPointSize(explanationFont.pointSize() - 1);
 
-    explanationLabel = new QLabel(tr("Here you can set some filters. You can filter "
-                                     "by content, author or application.\n\n"
+    explanationLabel = new QLabel(tr("Here you can set some rules for hiding or "
+                                     "highlighting stuff. "
+                                     "You can filter by content, author "
+                                     "or application.\n\n"
                                      "For instance, you can filter out messages posted by "
                                      "the application Open Farm Game, or which contain the "
-                                     "word NSFW in the message.")
-                                  + "\n\n"
-                                  + tr("At the moment, these filters only apply to "
-                                       "the Meanwhile feed."));
+                                     "word NSFW in the message. You could also highlight "
+                                     "messages that contain your name.")
+                                  + "\n");
     explanationLabel->setAlignment(Qt::AlignLeft | Qt::AlignTop);
     explanationLabel->setWordWrap(true);
     explanationLabel->setFont(explanationFont);
 
+
+    actionTypeComboBox = new QComboBox();
+    actionTypeComboBox->addItem(QIcon::fromTheme("edit-delete"),
+                                tr("Hide"));
+    actionTypeComboBox->addItem(QIcon::fromTheme("format-fill-color"),
+                                tr("Highlight"));
+
     filterTypeComboBox = new QComboBox();
     filterTypeComboBox->addItem(QIcon::fromTheme("accessories-text-editor"),
-                                tr("Content"));
+                                tr("Post Contents"));
     filterTypeComboBox->addItem(QIcon::fromTheme("user-identity"),
-                                tr("Author"));
+                                tr("Author ID"));
     filterTypeComboBox->addItem(QIcon::fromTheme("applications-other"),
                                 tr("Application"));
+    filterTypeComboBox->addItem(QIcon::fromTheme("accessories-text-editor"),
+                                tr("Activity Description"));
 
     filterWordsLineEdit = new QLineEdit();
     filterWordsLineEdit->setPlaceholderText(tr("Keywords..."));
@@ -81,11 +96,24 @@ FilterEditor::FilterEditor(PumpController *pumpController,
     connect(cancelButton, SIGNAL(clicked()),
             this, SLOT(hide()));
 
+    closeAction = new QAction(this);
+    closeAction->setShortcut(QKeySequence(Qt::Key_Escape));
+    connect(closeAction, SIGNAL(triggered()),
+            this, SLOT(hide()));
+    this->addAction(closeAction);
 
     // Layout
     topLayout = new QHBoxLayout();
+    topLayout->addWidget(actionTypeComboBox);
+    topLayout->addSpacing(2);
+    topLayout->addWidget(new QLabel(tr("if")));
+    topLayout->addSpacing(2);
     topLayout->addWidget(filterTypeComboBox);
-    topLayout->addWidget(filterWordsLineEdit);
+    topLayout->addSpacing(2);
+    topLayout->addWidget(new QLabel(tr("contains")));
+    topLayout->addSpacing(2);
+    topLayout->addWidget(filterWordsLineEdit, 1);
+    topLayout->addSpacing(8);
     topLayout->addWidget(addFilterButton);
 
     newFilterGroupBox = new QGroupBox(tr("&New Filter"));
@@ -134,23 +162,26 @@ void FilterEditor::loadFilters()
 
     foreach (QVariant filterItem, filtersList)
     {
+        int actionType = filterItem.toMap().value("action").toInt();
         int filterType = filterItem.toMap().value("type").toInt();
         QString filterWords = filterItem.toMap().value("text").toString();
 
         QVariantMap itemMap;
-        itemMap.insert("type", filterType);
-        itemMap.insert("text", filterWords);
+        itemMap.insert("action", actionType);
+        itemMap.insert("type",   filterType);
+        itemMap.insert("text",   filterWords);
 
-        QListWidgetItem *item = new QListWidgetItem(filterTypeComboBox->itemText(filterType)
-                                                    + " = "
-                                                    + filterWords);
+        QListWidgetItem *item = new QListWidgetItem(ruleString
+                                                    .arg(actionTypeComboBox->itemText(actionType))
+                                                    .arg(filterTypeComboBox->itemText(filterType))
+                                                    .arg(filterWords));
         item->setData(Qt::UserRole, itemMap);
 
         this->currentFiltersListWidget->addItem(item);
     }
 
-    // Let the PumpController know
-    this->pController->setFilters(filtersList);
+    // Let the FilterChecker know
+    this->fChecker->setFilters(filtersList);
 }
 
 
@@ -170,11 +201,14 @@ void FilterEditor::addFilter()
     }
 
     QVariantMap itemMap;
-    itemMap.insert("type", filterTypeComboBox->currentIndex());
-    itemMap.insert("text", words);
+    itemMap.insert("action", actionTypeComboBox->currentIndex());
+    itemMap.insert("type",   filterTypeComboBox->currentIndex());
+    itemMap.insert("text",   words);
 
-    QListWidgetItem *item = new QListWidgetItem(filterTypeComboBox->currentText()
-                                                + " = " + words);
+    QListWidgetItem *item = new QListWidgetItem(ruleString
+                                                .arg(actionTypeComboBox->currentText())
+                                                .arg(filterTypeComboBox->currentText())
+                                                .arg(words));
     item->setData(Qt::UserRole, itemMap);
 
     currentFiltersListWidget->addItem(item);
@@ -206,7 +240,7 @@ void FilterEditor::saveFilters()
     for (int counter = 0; counter != currentFiltersListWidget->count(); ++counter)
     {
         QListWidgetItem *item = currentFiltersListWidget->item(counter);
-        filtersList.append(item->data(Qt::UserRole));
+        filtersList.append(item->data(Qt::UserRole)); // item->data() holds a QVariantMap with the filter
     }
 
     qDebug() << "filtersList: " << this->filtersList;
@@ -214,8 +248,8 @@ void FilterEditor::saveFilters()
     QSettings settings;
     settings.setValue("Filters/currentFilters", filtersList);
 
-    // Send to PumpController too
-    this->pController->setFilters(filtersList);
+    // Send to the FilterChecker too
+    this->fChecker->setFilters(filtersList);
 
     this->hide();
 }
