@@ -22,49 +22,89 @@
 
 ImageViewer::ImageViewer(QString fileURI,
                          QString title,
-                         QWidget *parent) : QLabel(parent)
+                         QWidget *parent) : QWidget(parent)
 {
-    this->setMinimumSize(250, 100);
+    this->setMinimumSize(420, 320);
 
+    QString tooltipTitle;
     if (title.isEmpty())
     {
         title = "--";
     }
+    else
+    {
+        tooltipTitle = "<b><u>" + title + "</u></b><br><br>";
+    }
 
     fileURI.remove(0, 7); // remove "image:/" from filename URI
     this->originalFileURI = fileURI;
 
     QDesktopWidget desktopWidget;
-    QPixmap pixmap(originalFileURI);
-    QString resolution = QString("%1x%2").arg(pixmap.width()).arg(pixmap.height());
+    originalPixmap = QPixmap(originalFileURI);
+    QString resolution = QString("%1x%2")
+                         .arg(originalPixmap.width())
+                         .arg(originalPixmap.height());
 
-    this->setWindowTitle("Dianara - " + tr("Image") + ": " + title
-                         + " (" + resolution + ", "
-                         + MiscHelpers::fileSizeString(originalFileURI) +  ")");
+    QString imageDetails = resolution + ", "
+                           + MiscHelpers::fileSizeString(originalFileURI);
+
+    this->setWindowTitle(tr("Image") + ": " + title + " - Dianara");
     this->setWindowIcon(QIcon::fromTheme("folder-image"));
-    this->setAlignment(Qt::AlignCenter);
-    this->setToolTip(tr("ESC to close, secondary-click for options"));
+
+    imageLabel = new QLabel();
+    imageLabel->setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
+    imageLabel->setAlignment(Qt::AlignCenter);
+    imageLabel->setToolTip(tooltipTitle
+                           + tr("Resolution and file size") + ":<br>"
+                           + imageDetails
+                           + "<br><br>"
+                           + tr("ESC to close, secondary-click for options"));
 
 
-    // Resize pixmap according to desktop (screen) size
-    if (pixmap.height() > desktopWidget.height())
-    {
-        pixmap = pixmap.scaledToHeight(desktopWidget.height() - 100,
-                                       Qt::SmoothTransformation);
-    }
+    saveImageButton = new QPushButton(QIcon::fromTheme("document-save-as"),
+                                      tr("&Save As..."));
+    connect(saveImageButton, SIGNAL(clicked()),
+            this, SLOT(saveImage()));
 
-    if (pixmap.width() > desktopWidget.width())
-    {
-        pixmap = pixmap.scaledToWidth(desktopWidget.width() - 100,
-                                      Qt::SmoothTransformation);
-    }
+    infoLabel = new QLabel(imageDetails);
+    infoLabel->setAlignment(Qt::AlignCenter);
+
+    closeButton = new QPushButton(QIcon::fromTheme("window-close"),
+                                  tr("&Close"));
+    connect(closeButton, SIGNAL(clicked()),
+            this, SLOT(close()));
 
-    this->setPixmap(pixmap);
-    this->resize(pixmap.size());
 
     this->createContextMenu();
 
 
+    // Layout
+    buttonsLayout = new QHBoxLayout();
+    buttonsLayout->addWidget(saveImageButton);
+    buttonsLayout->addWidget(infoLabel, 1);
+    buttonsLayout->addWidget(closeButton);
+
+    mainLayout = new QVBoxLayout();
+    mainLayout->addWidget(imageLabel);
+    mainLayout->addLayout(buttonsLayout);
+    this->setLayout(mainLayout);
+
+
+    // Set initial window size according to image size and desktop (screen) size
+    int windowHeight = originalPixmap.height() + 100;
+    if (windowHeight > desktopWidget.height())
+    {
+        windowHeight = desktopWidget.height();
+    }
+
+    int windowWidth = originalPixmap.width() + 100;
+    if (windowWidth > desktopWidget.width())
+    {
+        windowWidth = desktopWidget.width();
+    }
+
+    this->resize(windowWidth, windowHeight);
+
     qDebug() << "ImageViewer created";
 }
 
@@ -75,9 +115,10 @@ ImageViewer::~ImageViewer()
 }
 
 
+
 void ImageViewer::createContextMenu()
 {
-    contextMenu = new QMenu("imaveViewerMenu");
+    contextMenu = new QMenu("imageViewerMenu");
 
 
     contextMenu->addAction(QIcon::fromTheme("document-save-as"),
@@ -91,33 +132,6 @@ void ImageViewer::createContextMenu()
 
 
 
-void ImageViewer::closeEvent(QCloseEvent *event)
-{
-    qDebug() << "ImageViewer::closeEvent()";
-    event->accept();
-
-    this->deleteLater();
-}
-
-
-void ImageViewer::keyPressEvent(QKeyEvent *event)
-{
-    if (event->key() == Qt::Key_Escape)
-    {
-        this->close();
-    }
-    else
-    {
-        event->accept();
-    }
-}
-
-
-void ImageViewer::contextMenuEvent(QContextMenuEvent *event)
-{
-    this->contextMenu->exec(event->globalPos());
-}
-
 
 /****************************************************************************/
 /************************************ SLOTS *********************************/
@@ -149,3 +163,60 @@ void ImageViewer::saveImage()
         }
     }
 }
+
+
+
+/****************************************************************************/
+/********************************** PROTECTED *******************************/
+/****************************************************************************/
+
+
+void ImageViewer::closeEvent(QCloseEvent *event)
+{
+    qDebug() << "ImageViewer::closeEvent()";
+    event->accept();
+
+    this->deleteLater();
+}
+
+void ImageViewer::resizeEvent(QResizeEvent *event)
+{
+    qDebug() << "ImageViewer::resizeEvent()";
+
+    int width = imageLabel->width() - 16;   // 16px as margin
+    if (width > originalPixmap.width())
+    {
+        width = originalPixmap.width();
+    }
+
+    int height = imageLabel->height() - 16;
+    if (height > originalPixmap.height())
+    {
+        height = originalPixmap.height();
+    }
+
+    imageLabel->setPixmap(originalPixmap.scaled(width, height,
+                                                Qt::KeepAspectRatio,
+                                                Qt::SmoothTransformation));
+
+    event->accept();
+}
+
+
+void ImageViewer::keyPressEvent(QKeyEvent *event)
+{
+    if (event->key() == Qt::Key_Escape)
+    {
+        this->close();
+    }
+    else
+    {
+        event->accept();
+    }
+}
+
+
+void ImageViewer::contextMenuEvent(QContextMenuEvent *event)
+{
+    this->contextMenu->exec(event->globalPos());
+}
