@@ -26,8 +26,8 @@ ListsManager::ListsManager(PumpController *pumpController,
     this->pController = pumpController;
     connect(pController, SIGNAL(personListReceived(QVariantList,QString)),
             this, SLOT(setPersonsInList(QVariantList,QString)));
-    connect(pController, SIGNAL(personAddedToList(QString,QString)),
-            this, SLOT(addPersonItemToList(QString,QString)));
+    connect(pController, SIGNAL(personAddedToList(QString,QString,QString)),
+            this, SLOT(addPersonItemToList(QString,QString,QString)));
     connect(pController, SIGNAL(personRemovedFromList(QString)),
             this, SLOT(removePersonItemFromList(QString)));
 
@@ -36,6 +36,8 @@ ListsManager::ListsManager(PumpController *pumpController,
     this->listsTreeWidget = new QTreeWidget();
     listsTreeWidget->setColumnCount(2);
     listsTreeWidget->setHeaderLabels(QStringList() << tr("Name") << tr("Members"));
+    listsTreeWidget->setSortingEnabled(true);
+    listsTreeWidget->sortByColumn(0, Qt::AscendingOrder);
     listsTreeWidget->setAlternatingRowColors(true);
     listsTreeWidget->setSizePolicy(QSizePolicy::Minimum,
                                    QSizePolicy::MinimumExpanding);
@@ -88,8 +90,8 @@ ListsManager::ListsManager(PumpController *pumpController,
                                           PeopleWidget::StandaloneWidget,
                                           this->pController,
                                           this);
-    connect(peopleWidget, SIGNAL(addButtonPressed(QString)),
-            this, SLOT(addPerson(QString)));
+    connect(peopleWidget, SIGNAL(addButtonPressed(QIcon,QString)),
+            this, SLOT(addPerson(QIcon,QString)));
 
 
     // Layout
@@ -136,6 +138,7 @@ void ListsManager::setListsList(QVariantList listsList)
     this->personListsUrlList.clear();
 
     QString listName;
+    QString listDescription;
     QString listMembersCount;
     QVariant listId;
 
@@ -143,13 +146,21 @@ void ListsManager::setListsList(QVariantList listsList)
     {
         QVariantMap listMap = list.toMap();
         listName = listMap.value("displayName").toString();
+        listDescription = listMap.value("content").toString();
+        if (!listDescription.isEmpty())
+        {
+            listDescription.append("<b></b>"); // HTMLize it so it gets wordwrapped
+        }
+
         listMembersCount = listMap.value("members").toMap().value("totalItems").toString();
         listId = listMap.value("id");
         qDebug() << "list ID:" << listId.toString();
 
         QTreeWidgetItem *listItem = new QTreeWidgetItem();
         listItem->setText(0, listName);
+        listItem->setToolTip(0, listDescription);
         listItem->setText(1, listMembersCount);
+        listItem->setToolTip(1, listDescription);
         listItem->setData(0, Qt::UserRole, listId);
 
 
@@ -166,11 +177,36 @@ void ListsManager::setListsList(QVariantList listsList)
     }
 
 
+    // this->listsTreeWidget->expandAll();
     this->listsTreeWidget->resizeColumnToContents(0);
 }
 
 
 
+QTreeWidgetItem *ListsManager::createPersonItem(QString id, QString name,
+                                                QString avatarFile)
+{
+    QTreeWidgetItem *personItem = new QTreeWidgetItem();
+
+    avatarFile = MiscHelpers::getCachedAvatarFilename(avatarFile);
+    QPixmap avatarPixmap = QPixmap(avatarFile);
+    if (avatarPixmap.isNull())
+    {
+        personItem->setIcon(0, QIcon::fromTheme("user-identity"));
+    }
+    else
+    {
+        personItem->setIcon(0, QIcon(avatarPixmap));
+    }
+    personItem->setText(0, name);
+    personItem->setText(1, id);
+
+
+    return personItem;
+}
+
+
+
 /****************************************************************************/
 /********************************* SLOTS ************************************/
 /****************************************************************************/
@@ -202,6 +238,7 @@ void ListsManager::setPersonsInList(QVariantList personList, QString listUrl)
 
     foreach (QVariant personVariant, personList)
     {
+        // FIXME: this should use ASPerson()
         QVariantMap personMap = personVariant.toMap();
 
         QString id = personMap.value("id").toString();
@@ -211,15 +248,11 @@ void ListsManager::setPersonsInList(QVariantList personList, QString listUrl)
         }
 
         QString name = personMap.value("displayName").toString();
-
-
-        QTreeWidgetItem *childItem = new QTreeWidgetItem();
-        childItem->setIcon(0, QIcon::fromTheme("user-identity"));
-        childItem->setText(0, name);
-        childItem->setText(1, id);
+        QString avatar = personMap.value("image").toMap().value("url").toString();
 
         if (listItem != 0) // Ensure it's initialized!
         {
+            QTreeWidgetItem *childItem = this->createPersonItem(id, name, avatar);
             listItem->addChild(childItem);
         }
     }
@@ -264,6 +297,9 @@ void ListsManager::deleteList()
 
     if (confirmation == 0)
     {
+        // Avoid the possibility of deleting twice!
+        this->deleteListButton->setDisabled(true);
+
         QString listId = this->listsTreeWidget->currentItem()->data(0, Qt::UserRole).toString();
 
         this->pController->deletePersonList(listId);
@@ -301,7 +337,7 @@ void ListsManager::enableDisableDeleteButtons()
 
 void ListsManager::showAddPersonDialog()
 {
-    if (listsTreeWidget->currentItem() == 0)
+    if (listsTreeWidget->currentItem() == 0) // No list selected
     {
         return;
     }
@@ -313,8 +349,9 @@ void ListsManager::showAddPersonDialog()
 }
 
 
-void ListsManager::addPerson(QString contactString)
+void ListsManager::addPerson(QIcon icon, QString contactString)
 {
+    Q_UNUSED(icon)
     this->peopleWidget->hide();
 
 
@@ -392,27 +429,27 @@ void ListsManager::removePerson()
  *
  */
 void ListsManager::addPersonItemToList(QString personId,
-                                       QString personName)
+                                       QString personName,
+                                       QString avatarUrl)
 {
     if (personId.startsWith("acct:"))
     {
         personId.remove(0,5);
     }
 
-    QTreeWidgetItem *newItem = new QTreeWidgetItem();
-    newItem->setIcon(0, QIcon::fromTheme("user-identity"));
-    newItem->setText(0, personName);
-    newItem->setText(1, personId);
+    QTreeWidgetItem *newItem = this->createPersonItem(personId,
+                                                      personName,
+                                                      avatarUrl);
 
     QTreeWidgetItem *selectedItem = this->listsTreeWidget->currentItem();
-    if (selectedItem->parent() == 0) // The list itself is selected
+    // When the list itself is NOT selected, but a child, point to the parent
+    if (selectedItem->parent() != 0)
     {
-        selectedItem->addChild(newItem);
-    }
-    else  // One of the members is selected, so add the child to the parent item
-    {
-        selectedItem->parent()->addChild(newItem);
-    }
+        selectedItem = selectedItem->parent();
+    }    
+    selectedItem->addChild(newItem);
+    // Update member counter
+    selectedItem->setText(1, QString("%1").arg(selectedItem->childCount()));
 
     this->setEnabled(true);
 }
@@ -422,7 +459,11 @@ void ListsManager::removePersonItemFromList(QString personId)
 {
     if (personId == "acct:" + listsTreeWidget->currentItem()->text(1))
     {
+        QTreeWidgetItem *parentList = listsTreeWidget->currentItem()->parent();
         delete listsTreeWidget->currentItem();
+
+        // Update member counter
+        parentList->setText(1, QString("%1").arg(parentList->childCount()));
     }
 
     this->setEnabled(true);
