@@ -50,6 +50,9 @@ public:
 
     void setListsList(QVariantList listsList);
 
+    QTreeWidgetItem *createPersonItem(QString id,
+                                      QString name,
+                                      QString avatarFile);
 
 signals:
 
@@ -62,10 +65,11 @@ public slots:
     void enableDisableDeleteButtons();
 
     void showAddPersonDialog();
-    void addPerson(QString contactString);
+    void addPerson(QIcon icon, QString contactString);
     void removePerson();
 
-    void addPersonItemToList(QString personId, QString personName);
+    void addPersonItemToList(QString personId, QString personName,
+                             QString avatarUrl);
     void removePersonItemFromList(QString personId);
 
 
