@@ -22,17 +22,26 @@
 
 
 MinorFeed::MinorFeed(PumpController *pumpController,
+                     FilterChecker *filterChecker,
                      QWidget *parent) : QFrame(parent)
 {
     this->pController = pumpController;
+    this->fChecker = filterChecker;
     this->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
 
     // Layout for the items
     itemsLayout = new QVBoxLayout();
+    itemsLayout->setContentsMargins(0, 0, 0, 0);
     itemsLayout->setAlignment(Qt::AlignTop);
 
+    // Separator frame, to mark where new items end
+    separatorFrame = new QFrame();
+    separatorFrame->setFrameStyle(QFrame::HLine);
+    separatorFrame->setMinimumHeight(24);
+
     // Button to get more items (older stuff)
-    getMoreButton = new QPushButton(QIcon::fromTheme("list-add"),
+    getMoreButton = new QPushButton(QIcon::fromTheme("list-add",
+                                                     QIcon(":/images/list-add.png")),
                                     tr("Get More"));
     getMoreButton->setFlat(true);
     getMoreButton->setToolTip(tr("Get older activities"));
@@ -42,14 +51,20 @@ MinorFeed::MinorFeed(PumpController *pumpController,
     // Set disabled initially; will be enabled when contents are set
     getMoreButton->setDisabled(true);
 
+
     // Main layout
     mainLayout = new QVBoxLayout();
+    mainLayout->setContentsMargins(1, 2, 1, 2);
     mainLayout->setAlignment(Qt::AlignTop);
     mainLayout->addLayout(itemsLayout);
     mainLayout->addWidget(getMoreButton);
     this->setLayout(mainLayout);
 
 
+    // Set the 5 colors empty, on start, since they're required when creating the demo
+    // item feed, and they're not set at this point
+    highlightColorsList << "" << "" << "" << "" << "";
+
 
     // Demo activity, stating that there's nothing to show
     QVariantMap demoGenerator;
@@ -64,7 +79,8 @@ MinorFeed::MinorFeed(PumpController *pumpController,
     ASActivity *demoActivity = new ASActivity(demoActivityMap, this);
 
     MinorFeedItem *demoFeedItem = new MinorFeedItem(demoActivity,
-                                                    "",
+                                                    this->highlightColorsList,
+                                                    false, // Not highlighted by filter
                                                     this->pController);
     this->itemsLayout->addWidget(demoFeedItem);
     this->itemsInFeed.append(demoFeedItem);
@@ -72,6 +88,7 @@ MinorFeed::MinorFeed(PumpController *pumpController,
     this->feedOffset = 0;
     this->newItemsCount = 0;
 
+    // Remember what was the newest activity last time
     QSettings settings;
     settings.beginGroup("MinorFeedState");
     this->previousNewestActivityId = settings.value("previousNewestItemId", "").toString();
@@ -93,6 +110,12 @@ MinorFeed::~MinorFeed()
 }
 
 
+void MinorFeed::updateHighlightColors(QStringList colorList)
+{
+    highlightColorsList = colorList;
+}
+
+
 void MinorFeed::clearContents()
 {
     qDebug() << "MinorFeed::clearContents()";
@@ -103,6 +126,9 @@ void MinorFeed::clearContents()
     }
 
     itemsInFeed.clear();
+
+    itemsLayout->removeWidget(separatorFrame);
+    separatorFrame->hide();
 }
 
 
@@ -114,6 +140,18 @@ void MinorFeed::markAllAsRead()
     }
 }
 
+/*
+ * Update fuzzy timestamps in all items
+ *
+ */
+void MinorFeed::updateFuzzyTimestamps()
+{
+    foreach (MinorFeedItem *feedItem, itemsInFeed)
+    {
+        feedItem->setFuzzyTimeStamp();
+    }
+}
+
 
 
 /*******************************************************************************/
@@ -157,96 +195,27 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
     qApp->processEvents(); // This also acts as "scroll to top", incidentally
 
 
-    QStringList filteredContent;
-    QStringList filteredAuthor;
-    QStringList filteredGenerator;
-
-    // Define filters
-    foreach (QVariant filter, pController->getCurrentFilters())
-    {
-        int filterType = filter.toMap().value("type").toInt();
-        QString filterText = filter.toMap().value("text").toString();
-
-        switch (filterType)
-        {
-        case 0: // content
-            filteredContent.append(filterText);
-            break;
-
-        case 1: // author
-            filteredAuthor.append(filterText);
-            break;
-
-        case 2: // application (generator)
-            filteredGenerator.append(filterText);
-            break;
-
-        default:
-            break;
-        }
-    }
-
-
-
-    QString activityActorAvatarUrl;
-    QString activityActorAvatarFilename;
-
-    QString activityContent;
-    QString activityAuthor;
-    QString activityGenerator;
-
-    bool filtered;
 
+    bool itemHighlightedByFilter;
+    bool itemIsNew;    
     bool allNewItemsCounted = false;
     QString newestActivityId; // To store the activity ID for the newest item in the feed
                               // so we can know how many new items we receive next time
 
+    newHighlightedItemsCount = 0;
+
 
     foreach (QVariant activityVariant, activitiesList)
     {
+        itemIsNew = false;
+
         ASActivity *activity = new ASActivity(activityVariant.toMap(), this);
 
-        ////////////////////////////////////////////////////////////// Filtering Start
-        filtered = false;  // Innocent until proven guilty!
-
-        activityContent = activity->getContent();
-        foreach (QString content, filteredContent)
-        {
-            if (activityContent.contains(content))
-            {
-                qDebug() << "Filtering item because of Content:" << content;
-                filtered = true;
-            }
-        }
-
-        activityAuthor = activity->getAuthorName();
-        foreach (QString author, filteredAuthor)
-        {
-            if (activityAuthor.contains(author))
-            {
-                qDebug() << "Filtering item because of Author:" << author;
-                filtered = true;
-            }
-        }
-
-        activityGenerator = activity->getGenerator();
-        foreach (QString generator, filteredGenerator)
-        {
-            if (activityGenerator.contains(generator))
-            {
-                qDebug() << "Filtering item because of Application (generator):" << generator;
-                filtered = true;
-            }
-        }
-        ////////////////////////////////////////////////////////////// Filtering End
-
-        bool itemIsNew;
+        int filtered = fChecker->validateActivity(activity);
 
         // If there is no reason to filter out the item, add it to the feed
-        if (!filtered)
+        if (filtered != FilterChecker::FilterOut)
         {
-            itemIsNew = false;
-
             // Determine which activities are new
             if (newestActivityId.isEmpty()) // Only first time, for newest item
             {
@@ -266,11 +235,16 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
                 if (activity->getId() == this->previousNewestActivityId)
                 {
                     allNewItemsCounted = true;
+                    if (newItemsCount > 0)
+                    {
+                        this->itemsLayout->addWidget(separatorFrame);
+                        separatorFrame->show();
+                    }
                 }
                 else
                 {
                     // If activity is not ours, add it to the count
-                    if (activity->getAuthorId() != pController->currentUserId())
+                    if (activity->author()->getId() != pController->currentUserId())
                     {
                         ++newItemsCount;
                         itemIsNew = true;
@@ -279,22 +253,29 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
             }
 
 
-
-            activityActorAvatarUrl = activity->getAuthorAvatar();
-            activityActorAvatarFilename = MiscHelpers::getCachedAvatarFilename(activityActorAvatarUrl);
-            if (!QFile::exists(activityActorAvatarFilename))
+            if (filtered == FilterChecker::Highlight) // kinda TMP
             {
-                this->pController->getAvatar(activityActorAvatarUrl);
+                itemHighlightedByFilter = true;
+            }
+            else
+            {
+                itemHighlightedByFilter = false;
             }
 
             MinorFeedItem *newFeedItem = new MinorFeedItem(activity,
-                                                           activityActorAvatarFilename,
+                                                           this->highlightColorsList,
+                                                           itemHighlightedByFilter,
                                                            this->pController);
             if (itemIsNew)
             {
                 newFeedItem->setItemAsNew(true);
                 connect(newFeedItem, SIGNAL(itemRead()),
                         this, SLOT(decreaseNewItemsCount()));
+
+                if (newFeedItem->getItemHighlightType() != -1)
+                {
+                    ++this->newHighlightedItemsCount;
+                }
             }
             this->itemsLayout->addWidget(newFeedItem);
             this->itemsInFeed.append(newFeedItem);
@@ -309,6 +290,7 @@ void MinorFeed::setFeedContents(QVariantList activitiesList)
 
     this->previousNewestActivityId = newestActivityId;
     emit newItemsCountChanged(newItemsCount);
+    emit newItemsReceived(newItemsCount, newHighlightedItemsCount);
     qDebug() << "Meanwhile feed updated; New items:" << newItemsCount;
 
     this->getMoreButton->setEnabled(true);
