@@ -35,6 +35,9 @@
 #include <QStandardPaths>
 #endif
 
+#include <magic.h>
+
+
 #include <QDebug>
 
 class MiscHelpers : public QObject
@@ -47,7 +50,7 @@ public:
     static QString getCachedAvatarFilename(QString url);
     static QString getCachedImageFilename(QString url);
 
-    static QString getImageContentType(QString fileURI);
+    static QString getFileMimeType(QString fileUri);
     static int getImageWidth(QString fileURI);
 
     static QString fixLongName(QString name);
