@@ -25,6 +25,8 @@
 #include <QHBoxLayout>
 #include <QVBoxLayout>
 #include <QTextBrowser>
+//#include <QTextBlock>
+//#include <QAbstractTextDocumentLayout>
 #include <QLabel>
 #include <QIcon>
 #include <QPixmap>
@@ -36,6 +38,7 @@
 #include <QVariantList>
 #include <QMessageBox>
 #include <QTimer>
+#include <QSettings>
 
 #include <QDebug>
 
@@ -46,27 +49,28 @@
 #include "commenterblock.h"
 #include "imageviewer.h"
 #include "asactivity.h"
-
+#include "avatarbutton.h"
 
 class Post : public QFrame
 {
     Q_OBJECT
 
 public:
-    Post(PumpController *pumpController,
-         ASActivity *activity,
-         bool isStandalone = false,
-         QWidget *parent = 0);
+    Post(ASActivity *activity,
+         QStringList highlightColors,
+         bool highlightedByFilter,
+         bool isStandalone,
+         PumpController *pumpController,
+         QWidget *parent);
     ~Post();
 
-    void createAvatarMenu();
-    void setFollowUnfollow();
 
     void setMinMaxHeight(int newMinMaxHeight);
 
     void setPostContents();
     void setPostHeight();
-    void enqueueImageForDownload(QString url);
+    void resetResizesCount();
+    void getPendingImages();
 
     QString likesURL();
     void setLikes(QVariantList likesList, int likesCount=-1);
@@ -84,6 +88,18 @@ public:
     void setPostAsNew();
     void setPostAsRead();
 
+    int getHighlightType();
+
+    void setFuzzyTimestamps();
+
+    enum PostHightlightType
+    {
+        NoHighlight = -1,
+        MessageForUserHighlight,
+        OwnMessageHighlight,
+        FilterRulesHighlight
+    };
+
 
 signals:
     void postRead();
@@ -99,7 +115,9 @@ public slots:
     void getAllLikes();
 
     void commentOnPost();
-    void sendComment(QString comment);
+    void sendComment(QString commentText);
+    void updateComment(QString commentId, QString commentText);
+
     void getAllComments();
     void setAllComments(QVariantList commentsList,
                         QString originatingPostUrl);
@@ -112,22 +130,18 @@ public slots:
     void deletePost();
 
     void openClickedURL(QUrl url);
-    void showHighlightedURL(QUrl url);
+    void showHighlightedUrl(QString url);
+
 
-    void openAuthorProfileInBrowser();
     void openPostInBrowser();
     void copyPostUrlToClipboard();
-
-    void followUser();
-    void unfollowUser();
-
     void normalizeTextFormat();
 
     void triggerResize();
 
-    void redrawAvatar(QString avatarUrl, QString avatarFilename);
     void redrawImages(QString imageUrl);
 
+
 protected:
     virtual void resizeEvent(QResizeEvent *event);
     virtual void mousePressEvent(QMouseEvent *event);
@@ -141,24 +155,26 @@ private:
     QVBoxLayout *leftColumnLayout;
     QVBoxLayout *rightColumnLayout;
 
+    QFrame *leftColumnFrame;
+
     QString postId;
     QString postType;
     QString postUrl;
-    QString postAuthorId;
     QString postAuthorName;
-    QString postAuthorUrl;
-    QString postAuthorAvatarUrl;
-    bool postAuthorFollowed;
 
     QString postSharedById;
+    QString sharedByTooltipString;
+    QString sharedByGeneratorString;
+    QString shareTime;
 
     bool postIsOwn;
     bool postIsUnread;
+    int highlightType;
 
-    QPushButton *postAuthorAvatarButton;
-    QMenu *avatarMenu;
-    QAction *avatarMenuIdAction;
-    QAction *avatarMenuFollowAction;
+    AvatarButton *postAuthorAvatarButton;
+    QAction *openPostInBrowserAction;
+    QAction *copyPostUrlAction;
+    QAction *normalizeTextAction;
 
     QLabel *postAuthorNameLabel;
     QLabel *postCreatedAtLabel;
@@ -173,9 +189,7 @@ private:
 
 
     QLabel *postTitleLabel;
-    QVBoxLayout *postTextLayout;
     QTextBrowser *postText;
-    QLabel *highlightedUrlLabel;
 
     QHBoxLayout *buttonsLayout;
     QPushButton *likeButton;
@@ -187,18 +201,29 @@ private:
 
     QString postTitle;
     QString postImageUrl;
+    QString postAudioUrl;
+    QString postVideoUrl;
+    QString postFileUrl;
     QString postOriginalText;
 
+    QString postCreatedAtString;
+    QString postUpdatedAtString;
+
     int postWidth;
     int minMaxHeight;
 
+    int resizesCount;
+
     QString postLikesUrl;
     QString postCommentsUrl;
     QString postSharesUrl;
+
     QStringList pendingImagesList;
 
     bool standalone;
 
+    QString seeFullImageString;
+
     PumpController *pController;
     CommenterBlock *commenter;
 };
