@@ -33,6 +33,7 @@ Publisher::Publisher(PumpController *pumpController,
     connect(pController, SIGNAL(listsListReceived(QVariantList)),
             this, SLOT(updateListsMenus(QVariantList)));
 
+    this->postType = "note";
 
     this->editingMode = false; // False unless set from setEditingMode
                                // after clicking "Edit" in a post
@@ -56,7 +57,7 @@ Publisher::Publisher(PumpController *pumpController,
     titleLineEdit = new QLineEdit();
     titleLineEdit->setSizePolicy(QSizePolicy::MinimumExpanding,
                                  QSizePolicy::Maximum);
-    titleLineEdit->setPlaceholderText(tr("Optional title for the post"));
+    titleLineEdit->setPlaceholderText(tr("Title for the post"));
     titleLineEdit->setToolTip(tr("Title for the post. Setting a title "
                                  "helps make the Meanwhile feed "
                                  "more informative.") + "<b></b>");
@@ -69,22 +70,18 @@ Publisher::Publisher(PumpController *pumpController,
     pictureLabel->hide();
 
 
-    pictureInfoLabel = new QLabel();
-    pictureInfoLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
+    mediaInfoLabel = new QLabel();
+    mediaInfoLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
 
-    selectPictureButton = new QPushButton(QIcon::fromTheme("insert-image"),
-                                          tr("Select Picture..."));
-    selectPictureButton->setSizePolicy(QSizePolicy::MinimumExpanding,
+    selectMediaButton = new QPushButton();
+    selectMediaButton->setSizePolicy(QSizePolicy::MinimumExpanding,
                                        QSizePolicy::Maximum);
-    selectPictureButton->setToolTip(tr("Find the picture in your folders"));
-    connect(selectPictureButton, SIGNAL(clicked()),
-            this, SLOT(findPictureFile()));
-    selectPictureButton->hide();
+    selectMediaButton->hide();
 
 
 
-    // Set default pixmap and "picture not set" message
-    this->setEmptyPictureData();
+    // Set default pixmap and "media not set" message
+    this->setEmptyMediaData();
     lastUsedDirectory = QDir::homePath();
 
 
@@ -187,15 +184,39 @@ Publisher::Publisher(PumpController *pumpController,
     ccPublicFollowersLabel->setFont(audienceLabelsFont);  // Bold + italic
 
 
-    // To set the 'picture mode'
-    pictureButton = new QPushButton(QIcon::fromTheme("camera-photo"),
-                                    tr("Ad&d Picture"));
-    pictureButton->setToolTip(tr("Upload photo"));
-    connect(pictureButton, SIGNAL(clicked()),
+    // Menu to set the picture/audio/video modes, under an "Add..." button
+    addMediaImageAction = new QAction(QIcon::fromTheme("camera-photo"),
+                                      tr("Picture"), this);
+    connect(addMediaImageAction, SIGNAL(triggered()),
             this, SLOT(setPictureMode()));
 
+    addMediaAudioAction = new QAction(QIcon::fromTheme("audio-input-microphone"),
+                                      tr("Audio") + " (experimental)", this);
+    addMediaAudioAction->setDisabled(true); // Disabled for v1.2.0
+    connect(addMediaAudioAction, SIGNAL(triggered()),
+            this, SLOT(setAudioMode()));
 
-    // Sending status info label
+    addMediaVideoAction = new QAction(QIcon::fromTheme("camera-web"),
+                                      tr("Video") + " (experimental)", this);
+    addMediaVideoAction->setDisabled(true); // Disabled for v1.2.0
+    connect(addMediaVideoAction, SIGNAL(triggered()),
+            this, SLOT(setVideoMode()));
+
+
+    // The menu itself
+    addMediaMenu = new QMenu();
+    addMediaMenu->addAction(addMediaImageAction);
+    addMediaMenu->addAction(addMediaAudioAction);
+    addMediaMenu->addAction(addMediaVideoAction);
+
+    // The "add..." button holding the menu
+    addMediaButton = new QPushButton(QIcon::fromTheme("mail-attachment"),
+                                    tr("Ad&d..."));
+    addMediaButton->setToolTip(tr("Upload media, like pictures or videos"));
+    addMediaButton->setMenu(addMediaMenu);
+
+
+    // Info label for "sending" and other status information
     statusInfoLabel = new QLabel();
     statusInfoLabel->setSizePolicy(QSizePolicy::MinimumExpanding,
                                    QSizePolicy::Maximum);
@@ -207,13 +228,15 @@ Publisher::Publisher(PumpController *pumpController,
 
 
     // To send the post
-    postButton = new QPushButton(QIcon::fromTheme("mail-send"),
+    postButton = new QPushButton(QIcon::fromTheme("mail-send",
+                                                  QIcon(":/images/button-post.png")),
                                   tr("Post"));
     postButton->setToolTip(tr("Hit Control+Enter to post with the keyboard"));
     connect(postButton, SIGNAL(clicked()),
             this, SLOT(sendPost()));
 
-    cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel"),
+    cancelButton = new QPushButton(QIcon::fromTheme("dialog-cancel",
+                                                    QIcon(":/images/button-cancel.png")),
                                    tr("Cancel"));
     cancelButton->setToolTip(tr("Cancel the post"));
     connect(cancelButton, SIGNAL(clicked()),
@@ -233,21 +256,21 @@ Publisher::Publisher(PumpController *pumpController,
 
     mainLayout->addLayout(titleLayout,         0, 0, 1, 8, Qt::AlignTop | Qt::AlignLeft);
     mainLayout->addWidget(pictureLabel,        1, 0, 2, 4);
-    mainLayout->addWidget(pictureInfoLabel,    1, 4, 1, 4, Qt::AlignTop);
-    mainLayout->addWidget(selectPictureButton, 2, 4, 1, 4, Qt::AlignBottom | Qt::AlignLeft);
+    mainLayout->addWidget(mediaInfoLabel,      1, 4, 1, 4, Qt::AlignTop);
+    mainLayout->addWidget(selectMediaButton,   2, 4, 1, 4, Qt::AlignBottom | Qt::AlignLeft);
 
     mainLayout->addWidget(composerBox,         3, 0, 3, 8); // 3 rows, 8 columns
 
     mainLayout->addWidget(toSelectorButton,    6, 0, 1, 1, Qt::AlignLeft);
     mainLayout->addWidget(ccSelectorButton,    6, 1, 1, 1, Qt::AlignLeft);
-    mainLayout->addWidget(pictureButton,       6, 3, 1, 2, Qt::AlignLeft);
+    mainLayout->addWidget(addMediaButton,      6, 3, 1, 2, Qt::AlignLeft);
     mainLayout->addWidget(statusInfoLabel,     6, 5, 3, 2, Qt::AlignCenter);
     mainLayout->addWidget(postButton,          6, 7, 1, 1);
 
 
     // The 2 labels holding people's names, if any
-    mainLayout->addWidget(toAudienceLabel, 7, 0, 1, 1);
-    mainLayout->addWidget(ccAudienceLabel, 7, 1, 1, 1);
+    mainLayout->addWidget(toAudienceLabel,        7, 0, 1, 1);
+    mainLayout->addWidget(ccAudienceLabel,        7, 1, 1, 1);
 
     // The 2 labels holding "public/followers", if selected
     mainLayout->addWidget(toPublicFollowersLabel, 8, 0, 1, 1);
@@ -285,23 +308,18 @@ void Publisher::setDefaultPublicPosting(bool defaultPublicPosts)
 
 
 /*
- * Set default "no photo" pixmap and "picture not set" message
+ * Set default "no photo" pixmap and "picture/audio/video not set" message
  *
  * Clear the filename and content type variables too
  */
-void Publisher::setEmptyPictureData()
+void Publisher::setEmptyMediaData()
 {
-    pictureLabel->setPixmap(QIcon::fromTheme("image-x-generic")
-                            .pixmap(200, 150)
-                            .scaled(200, 150,
-                                    Qt::IgnoreAspectRatio,
-                                    Qt::SmoothTransformation));
-    pictureLabel->setToolTip(tr("Picture not set"));
+    pictureLabel->setToolTip("");
 
-    pictureInfoLabel->clear();
+    mediaInfoLabel->clear();
 
-    this->pictureFilename.clear();
-    this->pictureContentType.clear();
+    this->mediaFilename.clear();
+    this->mediaContentType.clear();
 }
 
 
@@ -413,6 +431,17 @@ QMap<QString,QString> Publisher::getAudienceMap()
 }
 
 
+void Publisher::setMediaModeWidgets()
+{
+    this->addMediaButton->setDisabled(true);
+
+    this->pictureLabel->show();
+
+    this->mediaInfoLabel->show();
+    this->selectMediaButton->show();
+}
+
+
 
 /********************************************************************/
 /***************************** SLOTS ********************************/
@@ -499,7 +528,7 @@ void Publisher::setMinimumMode()
 
     this->statusInfoLabel->clear();
     statusInfoLabel->hide();
-    this->pictureButton->hide();
+    this->addMediaButton->hide();
     this->postButton->hide();
     this->cancelButton->hide();
 
@@ -507,13 +536,15 @@ void Publisher::setMinimumMode()
     // Clear formatting options like bold, italic and underline
     this->composerBox->setCurrentCharFormat(QTextCharFormat());
 
-    // Hide "picture mode" controls
+    // Hide "media mode" controls
     this->pictureLabel->hide();
-    this->pictureInfoLabel->hide();
-    this->selectPictureButton->hide();
+    this->mediaInfoLabel->hide();
+    this->selectMediaButton->hide();
+    this->selectMediaButton->disconnect(); // Disconnect from findPicture, findAudio...
 
-    this->setEmptyPictureData();
+    this->setEmptyMediaData();
 
+    this->postType = "note";
 
     // Clear "editing mode", restore stuff
     if (editingMode)
@@ -524,7 +555,7 @@ void Publisher::setMinimumMode()
         this->postButton->setText(tr("Post")); // Button text back to "Post" as usual
         this->toSelectorButton->setEnabled(true);
         this->ccSelectorButton->setEnabled(true);
-        this->pictureButton->setEnabled(true);
+        this->addMediaButton->setEnabled(true);
     }
 }
 
@@ -557,11 +588,11 @@ void Publisher::setFullMode()
     updatePublicFollowersLabels();
 
     // Avoid re-enabling the picture button when re-focusing publisher, but still in picture mode...
-    if (pictureButton->isHidden())
+    if (addMediaButton->isHidden())
     {
-        this->pictureButton->setEnabled(true); // If it wasn't hidden, don't re-enable
+        this->addMediaButton->setEnabled(true); // If it wasn't hidden, don't re-enable
     }
-    this->pictureButton->show();
+    this->addMediaButton->show();
     this->statusInfoLabel->show();
     this->postButton->show();
     this->cancelButton->show();
@@ -575,17 +606,80 @@ void Publisher::setFullMode()
 
 void Publisher::setPictureMode()
 {
-    this->pictureButton->setDisabled(true);
+    setMediaModeWidgets();
 
-    this->pictureLabel->show();
+    pictureLabel->setPixmap(QIcon::fromTheme("image-x-generic",
+                                             QIcon(":/images/attached-image.png"))
+                            .pixmap(200, 150)
+                            .scaled(200, 150,
+                                    Qt::IgnoreAspectRatio,
+                                    Qt::SmoothTransformation));
 
-    this->pictureInfoLabel->show();
-    this->selectPictureButton->show();
+    selectMediaButton->setIcon(QIcon::fromTheme("folder-image"));
+    selectMediaButton->setText(tr("Select Picture..."));
+    selectMediaButton->setToolTip(tr("Find the picture in your folders"));
+    connect(selectMediaButton, SIGNAL(clicked()),
+            this, SLOT(findPictureFile()));
+
+    pictureLabel->setToolTip(tr("Picture not set"));
+
+    postType = "image";
 
     this->findPictureFile(); // Show the open file dialog directly
 }
 
 
+void Publisher::setAudioMode()
+{
+    setMediaModeWidgets();
+
+    pictureLabel->setPixmap(QIcon::fromTheme("audio-x-generic",
+                                             QIcon(":/images/attached-audio.png"))
+                            .pixmap(200, 150)
+                            .scaled(200, 150,
+                                    Qt::IgnoreAspectRatio,
+                                    Qt::SmoothTransformation));
+
+    selectMediaButton->setIcon(QIcon::fromTheme("folder-sound"));
+    selectMediaButton->setText(tr("Select Audio File..."));
+    selectMediaButton->setToolTip(tr("Find the audio file in your folders"));
+    connect(selectMediaButton, SIGNAL(clicked()),
+            this, SLOT(findAudioFile()));
+
+    pictureLabel->setToolTip(tr("Audio file not set"));
+
+    postType = "audio";
+
+    this->findAudioFile(); // Show the open file dialog directly
+}
+
+
+void Publisher::setVideoMode()
+{
+    setMediaModeWidgets();
+
+    pictureLabel->setPixmap(QIcon::fromTheme("video-x-generic",
+                                             QIcon(":/images/attached-video.png"))
+                            .pixmap(200, 150)
+                            .scaled(200, 150,
+                                    Qt::IgnoreAspectRatio,
+                                    Qt::SmoothTransformation));
+
+    selectMediaButton->setIcon(QIcon::fromTheme("folder-video"));
+    selectMediaButton->setText(tr("Select Video..."));
+    selectMediaButton->setToolTip(tr("Find the video in your folders"));
+    connect(selectMediaButton, SIGNAL(clicked()),
+            this, SLOT(findVideoFile()));
+
+    pictureLabel->setToolTip(tr("Video not set"));
+
+    postType = "video";
+
+    this->findVideoFile(); // Show the open file dialog directly
+}
+
+
+
 /*
  * Set Publiser to edit mode, after user clicks on "Edit" in a post
  *
@@ -593,10 +687,9 @@ void Publisher::setPictureMode()
 void Publisher::setEditingMode(QString postID, QString postText,
                                QString postTitle)
 {
-    if (postButton->isVisible())  // FIXME, use a proper state variable
+    // Prevent the "Edit" option from destroying a post currently being composed!
+    if (editingMode)
     {
-        // TMPFIX; temporary way of preventing the "Edit" option
-        // from destroying a post currently being composed!
         QMessageBox::warning(this, tr("Error: Already composing"),
                              tr("You can't edit a post at this time, "
                                 "because a post is already being composed."));
@@ -616,7 +709,7 @@ void Publisher::setEditingMode(QString postID, QString postText,
     this->postButton->setText(tr("Update"));
     this->toSelectorButton->setDisabled(true);
     this->ccSelectorButton->setDisabled(true);
-    this->pictureButton->setDisabled(true);
+    this->addMediaButton->setDisabled(true);
 
 
     this->statusInfoLabel->setText(tr("Editing post"));
@@ -835,7 +928,7 @@ void Publisher::updateCcListsFields(QAction *listAction)
 
 
 /*
- * Send the post (note, image...) to the server
+ * Send the post (note, image, audio...) to the server
  *
  */
 void Publisher::sendPost()
@@ -843,11 +936,11 @@ void Publisher::sendPost()
     qDebug() << "Publisher character count:" << composerBox->textCursor().document()->characterCount();
 
 
-    // If there's some text in the post, or attached picture, send it
+    // If there's some text in the post, or attached media, send it
     if (composerBox->textCursor().document()->characterCount() > 1     // kinda tmp
-      || !pictureFilename.isEmpty())
+      || !mediaFilename.isEmpty())
     {
-        QString postTitle = this->titleLineEdit->text();
+        QString postTitle = this->titleLineEdit->text().trimmed();
         postTitle.replace("\n", " "); // Post title could have newlines if copy-pasted
 
         QString cleanHtmlString = MiscHelpers::cleanupHtml(composerBox->toHtml());
@@ -863,7 +956,7 @@ void Publisher::sendPost()
         {
             this->statusInfoLabel->setText(tr("Posting..."));
 
-            if (this->pictureFilename.isEmpty())
+            if (postType == "note")
             {
                 this->pController->postNote(audienceMap,
                                             cleanHtmlString,
@@ -871,11 +964,12 @@ void Publisher::sendPost()
             }
             else
             {
-                this->pController->postImage(audienceMap,
+                this->pController->postMedia(audienceMap,
                                              cleanHtmlString,
                                              postTitle,
-                                             pictureFilename,
-                                             pictureContentType);
+                                             mediaFilename,
+                                             postType,
+                                             mediaContentType);
             }
         }
         else
@@ -889,7 +983,7 @@ void Publisher::sendPost()
     else
     {
         this->statusInfoLabel->setText(tr("Post is empty."));
-        qDebug() << "Can't send post: text is empty, and no picture";
+        qDebug() << "Can't send post: text is empty, and no media";
     }
 }
 
@@ -902,7 +996,6 @@ void Publisher::sendPost()
 void Publisher::findPictureFile()
 {
     QString filename;
-
     filename = QFileDialog::getOpenFileName(this, tr("Select one image"),
                                             this->lastUsedDirectory,
                                             tr("Image files") + "(*.png *.jpg *.jpeg *.gif);;"
@@ -911,12 +1004,12 @@ void Publisher::findPictureFile()
     if (!filename.isEmpty())
     {
         qDebug() << "Selected" << filename << "for upload";
-        this->pictureLabel->setToolTip(filename);
 
-        this->pictureContentType = MiscHelpers::getImageContentType(filename);
-        if (!pictureContentType.isEmpty())
+        this->mediaContentType = MiscHelpers::getFileMimeType(filename);
+        if (!mediaContentType.isEmpty())
         {
-            this->pictureFilename = filename;
+            this->pictureLabel->setToolTip(filename);
+            this->mediaFilename = filename;
 
             QPixmap imagePixmap = QPixmap(filename);
             // FIXME: check imagePixmap.isNull() ?
@@ -928,7 +1021,7 @@ void Publisher::findPictureFile()
             qDebug() << "last used directory:" << lastUsedDirectory;
 
 
-            pictureInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
+            mediaInfoLabel->setText(QString("<b>" +tr("Resolution") + ":</b> %1x%2"
                                       "<br>"
                                       "<b>" + tr("Size") + ":</b> %3")
                                       .arg(imagePixmap.width())
@@ -937,16 +1030,100 @@ void Publisher::findPictureFile()
         }
         else
         {
-            // In the future, avoid this by using libmagic or similar
             qDebug() << "Unknown image format; Extension is probably wrong";
             QMessageBox::warning(this, tr("Invalid image"),
                                  tr("The image format cannot be detected.\n"
                                     "The extension might be wrong, like a GIF "
                                     "image renamed to image.jpg or similar."));
 
-            this->pictureContentType.clear();
+            this->mediaContentType.clear();
+            this->mediaFilename.clear();
+            this->pictureLabel->setToolTip("");
+        }
+    }
+}
 
-            this->pictureFilename.clear();
+
+
+
+void Publisher::findAudioFile()
+{
+    QString filename;
+    filename = QFileDialog::getOpenFileName(this, tr("Select one audio file"),
+                                            this->lastUsedDirectory,
+                                            tr("Audio files") + "(*.ogg *.flac *.mp3 *.wav);;"
+                                            + tr("All files") + " (*)");
+
+    if (!filename.isEmpty())
+    {
+        qDebug() << "Selected" << filename << "for upload";
+
+        this->mediaContentType = MiscHelpers::getFileMimeType(filename);
+        if (!mediaContentType.isEmpty())
+        {
+            this->pictureLabel->setToolTip(filename);
+            this->mediaFilename = filename;
+
+            QFileInfo fileInfo(filename);
+            this->lastUsedDirectory = fileInfo.path();
+            qDebug() << "last used directory:" << lastUsedDirectory;
+
+
+            mediaInfoLabel->setText(QString("<b>" + tr("Size") + ":</b> %1")
+                                    .arg(MiscHelpers::fileSizeString(filename)));
+        }
+        else
+        {
+            qDebug() << "Unknown audio format; Extension is probably wrong";
+            QMessageBox::warning(this, tr("Invalid audio file"),
+                                 tr("The audio format cannot be detected."));
+
+            this->mediaContentType.clear();
+            this->mediaFilename.clear();
+            this->pictureLabel->setToolTip("");
+        }
+    }
+}
+
+
+
+
+void Publisher::findVideoFile()
+{
+    QString filename;
+    filename = QFileDialog::getOpenFileName(this, tr("Select one video file"),
+                                            this->lastUsedDirectory,
+                                            tr("Video files")
+                                            + "(*.avi *.mpg *.mpeg *.ogm *.mkv *.mp4);;"
+                                            + tr("All files") + " (*)");
+
+    if (!filename.isEmpty())
+    {
+        qDebug() << "Selected" << filename << "for upload";
+
+        this->mediaContentType = MiscHelpers::getFileMimeType(filename);
+        if (!mediaContentType.isEmpty())
+        {
+            this->pictureLabel->setToolTip(filename);
+            this->mediaFilename = filename;
+
+            QFileInfo fileInfo(filename);
+            this->lastUsedDirectory = fileInfo.path();
+            qDebug() << "last used directory:" << lastUsedDirectory;
+
+
+            mediaInfoLabel->setText(QString("<b>" + tr("Size") + ":</b> %1")
+                                    .arg(MiscHelpers::fileSizeString(filename)));
+        }
+        else
+        {
+            qDebug() << "Unknown video format; Extension is probably wrong";
+            QMessageBox::warning(this, tr("Invalid video file"),
+                                 tr("The video format cannot be detected."));
+
+            this->mediaContentType.clear();
+            this->mediaFilename.clear();
+            this->pictureLabel->setToolTip("");
         }
     }
 }
