@@ -49,11 +49,21 @@ public:
     ~Publisher();
 
     void setDefaultPublicPosting(bool defaultPublicPosts);
-    void setEmptyPictureData();
+    void setEmptyMediaData();
     void updatePublicFollowersLabels();
 
     QMap<QString,QString> getAudienceMap();
 
+    void setMediaModeWidgets();
+
+    enum PostTypes
+    {
+        NoteType,
+        ImageType,
+        AudioType,
+        VideoType
+    };
+
 signals:
 
 
@@ -61,6 +71,8 @@ public slots:
     void setMinimumMode();
     void setFullMode();
     void setPictureMode();
+    void setAudioMode();
+    void setVideoMode();
 
     void setEditingMode(QString postID,
                         QString postText,
@@ -85,6 +97,8 @@ public slots:
     void sendPost();
 
     void findPictureFile();
+    void findAudioFile();
+    void findVideoFile();
 
 
 private:
@@ -93,8 +107,8 @@ private:
 
     QLabel *titleLabel;
     QLineEdit *titleLineEdit;
-    QLabel *pictureInfoLabel;
-    QPushButton *selectPictureButton;
+    QLabel *mediaInfoLabel;
+    QPushButton *selectMediaButton;
     QLabel *pictureLabel;
 
     QPushButton *toolsButton;
@@ -132,16 +146,22 @@ private:
     bool defaultPublicPosting;
 
 
-    QPushButton *pictureButton;
+    QPushButton *addMediaButton;
+    QMenu *addMediaMenu;
+    QAction *addMediaImageAction;
+    QAction *addMediaAudioAction;
+    QAction *addMediaVideoAction;
+
     QLabel *statusInfoLabel;
 
     QPushButton *postButton;
     QPushButton *cancelButton;
 
 
-    QString pictureFilename;
-    QString pictureContentType;
+    QString mediaFilename;
+    QString mediaContentType;
     QString lastUsedDirectory;
+    QString postType;
 
     bool editingMode;
     QString editingPostId;
