@@ -25,7 +25,7 @@ PumpController::PumpController(QObject *parent) :  QObject(parent)
     connect(&nam, SIGNAL(destroyed()),
             this, SLOT(tmpfixmeNotifyQNAMdestroyed()));
 
-    this->userAgentString = "Dianara/1.1";
+    this->userAgentString = "Dianara/1.2.0";
 
     this->postsPerPageMain = 20;
     this->postsPerPageOther = 10;
@@ -55,8 +55,6 @@ PumpController::PumpController(QObject *parent) :  QObject(parent)
         qDebug() << "Using token" << token;
         qDebug() << "And token secret" << tokenSecret;
     }
-    emit this->authorizationStatusChanged(isApplicationAuthorized);
-
 
 
 
@@ -112,19 +110,6 @@ void PumpController::setUpdatesToTimelineBlocked(bool blocked)
 }
 
 
-void PumpController::setFilters(QVariantList filterList)
-{
-    this->currentFilters = filterList;
-
-    qDebug() << "PumpController::setFilters()" << currentFilters;
-}
-
-QVariantList PumpController::getCurrentFilters()
-{
-    return this->currentFilters;
-}
-
-
 
 /*
  * Set new user ID (user@domain.tld) and clear OAuth-related tokens/secrets
@@ -168,6 +153,8 @@ void PumpController::setUserCredentials(QString userID)
     qDebug() << "New userID is:" << this->userId;
 
 
+    emit this->authorizationStatusChanged(isApplicationAuthorized);
+
     // This will call getUserProfile(), getContactList(), getMainTimeline(), etc.
     this->haveProfile = false;
     this->haveFollowing = false;
@@ -175,8 +162,12 @@ void PumpController::setUserCredentials(QString userID)
     this->havePersonLists = false;
 
     this->initialDataStep = 0;
-    this->initialDataTimer->start(2000);  // start 2 seconds after setting the ID
-                                          // (mainly on program startup)
+
+    if (isApplicationAuthorized)
+    {
+        this->initialDataTimer->start(2000);  // start 2 seconds after setting the ID
+                                              // (mainly on program startup)
+    }
 }
 
 
@@ -271,18 +262,64 @@ void PumpController::updateUserProfile(QString avatarUrl, QString fullName,
 }
 
 
-
-
-void PumpController::getAvatar(QString avatarURL)
+/*
+ * Add an avatar URL to the queue of pending
+ *
+ */
+void PumpController::enqueueAvatarForDownload(QString url)
 {
-    if (avatarURL.isEmpty())
+    if (QFile::exists(MiscHelpers::getCachedAvatarFilename(url))
+     || pendingAvatarsList.contains(url))
+    {
+        qDebug() << "PumpController() Using cached avatar, or it is pending download...";
+    }
+    else
+    {
+        this->pendingAvatarsList.append(url);
+        this->getAvatar(url);
+        qDebug() << "PumpController() Avatar not cached, downloading" << url;
+    }
+}
+
+
+
+/*
+ * Add the URL of an image to the queue of pending-download
+ *
+ * connect() signal/slot if necessary to refresh when done
+ *
+ */
+void PumpController::enqueueImageForDownload(QString url)
+{
+    if (QFile::exists(MiscHelpers::getCachedImageFilename(url))
+     || pendingImagesList.contains(url))
+    {
+        qDebug() << "PumpController::enqueueImageForDownload(), "
+                    "Using cached image, or requested image is pending download...";
+    }
+    else
+    {
+        this->pendingImagesList.append(url);
+        this->getImage(url);
+
+        qDebug() << "PumpController::enqueueImageForDownload(), "
+                    "image not cached, downloading" << url;
+    }
+}
+
+
+
+
+void PumpController::getAvatar(QString avatarUrl)
+{
+    if (avatarUrl.isEmpty())
     {
         return;
     }
 
-    qDebug() << "Getting avatar";
+    qDebug() << "Getting avatar" << avatarUrl;
 
-    QNetworkRequest avatarRequest(QUrl((const QString)avatarURL));
+    QNetworkRequest avatarRequest(QUrl((const QString)avatarUrl));
     avatarRequest.setRawHeader("User-Agent", userAgentString);
     avatarRequest.setAttribute(QNetworkRequest::User,
                                QVariant(AvatarRequest));
@@ -292,28 +329,33 @@ void PumpController::getAvatar(QString avatarURL)
 
 
 
-void PumpController::getImage(QString imageURL)
+void PumpController::getImage(QString imageUrl)
 {
-    if (imageURL.isEmpty())
+    if (imageUrl.isEmpty())
     {
         return;
     }
 
-    QNetworkRequest imageRequest = this->prepareRequest(imageURL, QOAuth::GET, ImageRequest);
-
+    QNetworkRequest imageRequest = this->prepareRequest(imageUrl,
+                                                        QOAuth::GET,
+                                                        ImageRequest);
     nam.get(imageRequest);
-    qDebug() << "imageRequest sent";
+    qDebug() << "getImage() imageRequest sent";
 }
 
 
 
 void PumpController::notifyAvatarStored(QString avatarUrl, QString avatarFilename)
 {
+    pendingAvatarsList.removeAll(avatarUrl);
+
     emit avatarStored(avatarUrl, avatarFilename);
 }
 
 void PumpController::notifyImageStored(QString imageUrl)
 {
+    pendingImagesList.removeAll(imageUrl);
+
     emit imageStored(imageUrl);
 }
 
@@ -387,6 +429,11 @@ void PumpController::updateInternalFollowingIdList(QStringList idList)
     this->followingIdList.append(idList);
 }
 
+void PumpController::removeFromInternalFollowingList(QString id)
+{
+    this->followingIdList.removeAll(id);
+}
+
 
 
 /*
@@ -585,14 +632,14 @@ void PumpController::getMainTimeline(int timelineOffset)
     QNetworkRequest timelineRequest = this->prepareRequest(url, QOAuth::GET,
                                                            MainTimelineRequest, paramMap);
 
-
+/*
     qDebug() << "===================================";
     qDebug() << "Getting timeline" << url;
     qDebug() << "with params:" << paramMap;
     qDebug() << "\nAuthorization Header:" << timelineRequest.rawHeader("Authorization");
     qDebug() << "\n\nFinal URL to retrieve:" << timelineRequest.url().toString();
     qDebug() << "===================================";
-
+*/
     nam.get(timelineRequest);
 }
 
@@ -692,6 +739,57 @@ void PumpController::getFavoritesTimeline(int timelineOffset)
 
 
 
+void PumpController::getTimeline(int timelineType, QString url)
+{
+    if (this->updatesToTimelineBlocked)
+    {
+        emit currentJobChanged(tr("Timeline update requested, "
+                                  "but updates are blocked."));
+        return;
+    }
+
+    emit currentJobChanged(tr("Getting timeline..."));
+
+    QStringList urlParts = url.split("?");
+    QString baseUrl = urlParts.at(0);
+    QStringList pageParameter;
+    if (urlParts.length() > 1)
+    {
+        pageParameter = urlParts.at(1).split("="); // since/before=http...
+    }
+
+    // 'since' or 'before'
+    QByteArray sinceBeforeWord;
+    sinceBeforeWord = pageParameter.at(0).toLocal8Bit();
+
+    // The url parameter
+    QByteArray sinceBeforeUrl;
+    sinceBeforeUrl = pageParameter.at(1).toLocal8Bit();
+                     //QByteArray::fromPercentEncoding(pageParameter.at(1).toLocal8Bit());
+
+
+    QOAuth::ParamMap paramMap;
+    paramMap.insert(sinceBeforeWord, sinceBeforeUrl); // since=http...
+    paramMap.insert("count", QString("%1").arg(this->postsPerPageMain).toLocal8Bit());
+
+
+    QNetworkRequest timelineRequest = this->prepareRequest(baseUrl, QOAuth::GET,
+                                                           MainTimelineRequest,
+                                                           paramMap);
+
+    qDebug() << "\n\n==== getTimeLine() =========================================";
+    qDebug() << "===================================";
+    qDebug() << "Getting timeline" << url;
+    qDebug() << "Base URL:" << baseUrl;
+    qDebug() << "with params:" << paramMap;
+    qDebug() << "\n\nAuthorization Header:" << timelineRequest.rawHeader("Authorization");
+    qDebug() << "\n\nFinal URL to retrieve:" << timelineRequest.url().toString();
+    qDebug() << "===================================";
+
+    nam.get(timelineRequest);
+}
+
+
 
 
 
@@ -797,7 +895,8 @@ QNetworkRequest PumpController::prepareRequest(QString url, QOAuth::HttpMethod m
     // Don't append inline parameters if they're empty, that can mess up things
     if (!paramMap.isEmpty())
     {
-        url.append(qoauth->inlineParameters(paramMap, QOAuth::ParseForInlineQuery));
+        url.append(qoauth->inlineParameters(paramMap,
+                                            QOAuth::ParseForInlineQuery));
     }
     request.setUrl(QUrl(url));
     // Only add Authorization header if we're requesting something in our server
@@ -809,7 +908,7 @@ QNetworkRequest PumpController::prepareRequest(QString url, QOAuth::HttpMethod m
     request.setRawHeader("User-Agent", userAgentString);
 
 
-    request.setAttribute(QNetworkRequest::User,  QVariant(requestType));
+    request.setAttribute(QNetworkRequest::User, QVariant(requestType));
 
     return request;
 }
@@ -900,6 +999,12 @@ QList<QVariantList> PumpController::processAudience(QMap<QString, QString> audie
 }
 
 
+void PumpController::showTransientMessage(QString message)
+{
+    emit transientStatusBarMessage(message);
+}
+
+
 
 /***************************************************************************/
 /***************************************************************************/
@@ -924,6 +1029,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
     QString replyErrorString = reply->errorString();
     QByteArray replyFirstLine = reply->readLine();
     QByteArray replyData = replyFirstLine + reply->readAll();
+    QString prettyLogMessage;
 
     qDebug() << "Request finished. HTTP code:" << httpCode;
     qDebug() << "Size:" << reply->size() << "bytes; URL:" << replyUrl;
@@ -948,19 +1054,25 @@ void PumpController::requestFinished(QNetworkReply *reply)
         {
             emit commentPostingFailed();
         }
+        else if (requestType == UpdateCommentRequest)
+        {
+            emit commentPostingFailed(); // also, kinda TMP
+        }
     }
 
 
     QString errorTypeString;
 
+
     switch (httpCode)
     {
-    // First, handle error codes
+    //////////////////////////////////////////////// First, handle error codes
     case 503:
         errorTypeString = tr("Service Unavailable", "HTTP 503 error string")
-                        +  " (503)";
+                        +  " (503) " + replyFirstLine;
 
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
 
         // if not just an image, it's important, so popup notification too
         if (requestType != ImageRequest && requestType != AvatarRequest)
@@ -974,9 +1086,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     case 500:
         errorTypeString = tr("Internal Server Error", "HTTP 500 error string")
-                        + " (500)";
+                        + " (500) " + replyFirstLine;
 
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
 
         // if not just an image, it's important, so popup notification too
         if (requestType != ImageRequest && requestType != AvatarRequest)
@@ -995,6 +1108,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
         errorTypeString = tr("Gone", "HTTP 410 error string")
                         + " (410)";
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 410: Gone.";
         qDebug() << "Data:  " << replyData;
         return;
@@ -1003,22 +1117,25 @@ void PumpController::requestFinished(QNetworkReply *reply)
         errorTypeString = tr("Not Found", "HTTP 404 error string")
                         + " (404)";
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 404: Not Found.";
         qDebug() << "Data:  " << replyData;
         return;
 
     case 403:
         errorTypeString = tr("Forbidden", "HTTP 403 error string")
-                        + " (403)";
+                        + " (403) " + replyFirstLine;
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 403: Forbidden.";
         qDebug() << "Data:  " << replyData;
         return;
 
     case 401:
         errorTypeString = tr("Unauthorized", "HTTP 401 error string")
-                        + " (401)";
+                        + " (401) " + replyFirstLine;
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 401: Unauthorized.";
         qDebug() << "Data:  " << replyData;
         return;
@@ -1026,9 +1143,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     case 400:
         errorTypeString = tr("Bad Request", "HTTP 400 error string")
-                        + " (400)";
+                        + " (400) " + replyFirstLine;
 
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
 
         // if not just an image, it's important, so popup notification too
         if (requestType != ImageRequest && requestType != AvatarRequest)
@@ -1045,6 +1163,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
         errorTypeString = tr("Moved Temporarily", "HTTP 302 error string")
                         + " (302)";
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 302: Moved Temporarily.";
         qDebug() << "Data:  " << replyData;
         return;
@@ -1053,20 +1172,24 @@ void PumpController::requestFinished(QNetworkReply *reply)
         errorTypeString = tr("Moved Permanently", "HTTP 301 error string")
                         + " (301)";
         emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "HTTP 301: Moved Permanently.";
         qDebug() << "Data:  " << replyData;
         return;
 
-    case 0:
-        emit currentJobChanged(tr("Error connecting to %1").arg(replyHost)
-                               + ": " + replyErrorString);
-        qDebug() << "Error connecting to" << replyUrl;
+    case 0: // Other kinds of network errors
+        errorTypeString = tr("Error connecting to %1").arg(replyHost);
+        emit currentJobChanged(errorTypeString + ": " + replyErrorString);
+        emit logMessage(errorTypeString + ": " + replyErrorString);
+
+        qDebug() << "Error connecting to" << replyHost;
         return;
 
     // Other HTTP codes
     default:
-        emit currentJobChanged(tr("Unhandled HTTP error code %1").arg(httpCode)
-                              + ": " + replyUrl);
+        errorTypeString = tr("Unhandled HTTP error code %1").arg(httpCode);
+        emit currentJobChanged(errorTypeString + ": " + replyUrl);
+        emit logMessage(errorTypeString, replyUrl);
         qDebug() << "Unhandled HTTP error " << httpCode;
         qDebug() << "Data:  " << replyData;
         return;
@@ -1132,7 +1255,9 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
         if (jsonParsedOK && jsonData.size() > 0)
         {
-            emit currentJobChanged(tr("Profile received."));
+            prettyLogMessage = tr("Profile received.");
+            emit currentJobChanged(prettyLogMessage);
+            emit logMessage(prettyLogMessage);
 
             QVariantMap profileMap = jsonData["profile"].toMap();
 
@@ -1189,7 +1314,8 @@ void PumpController::requestFinished(QNetworkReply *reply)
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
-            emit currentJobChanged(tr("Timeline received. Updating post list..."));
+            prettyLogMessage = tr("Timeline received. Updating post list...");
+            emit currentJobChanged(prettyLogMessage);
 
             jsonDataList = jsonData["items"].toList();
             qDebug() << "Number of items in timeline:" << jsonDataList.size();
@@ -1233,24 +1359,29 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     ////////////////////////////////////
     case PublishPostRequest:
-        emit currentJobChanged(tr("Post published successfully."));
+        prettyLogMessage = tr("Post published successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
 
             QString objectID = jsonData["object"].toMap().value("id").toString();
-            qDebug() << "Image post ID:" << objectID;
+            qDebug() << "Post ID:" << objectID;
 
-            if (jsonData["object"].toMap().value("objectType").toString() == "image")
+            QString objectType = jsonData["object"].toMap().value("objectType").toString();
+            if (objectType == "image"
+             || objectType == "audio"
+             || objectType == "video")
             {
-                // Update the image with title and description
-                this->postImageStepThree(objectID);
+                // Update the object with title and description
+                this->postMediaStepThree(objectID);
             }
             else
             {
-                // Not an image, notify "posted OK"
+                // Not a a media post, notify "posted OK"
                 emit postPublished();
-                qDebug() << "Non-image post published correctly";
+                qDebug() << "Non-media post published correctly";
             }
         }
         else
@@ -1264,7 +1395,9 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     case PublishAvatarRequest:
-        emit currentJobChanged(tr("Avatar published successfully."));
+        prettyLogMessage = tr("Avatar published successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
         if (jsonParsedOK && jsonData.size() > 0)
         {
             qDebug() << "JSON parsed OK";
@@ -1293,15 +1426,28 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     ////////////////////////////////////
     case UpdatePostRequest:
-        emit currentJobChanged(tr("Post updated successfully."));
+        prettyLogMessage = tr("Post updated successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
+
         emit postPublished();
         break;
 
+    case UpdateCommentRequest:
+        prettyLogMessage = tr("Comment updated successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
+
+        emit commentPosted();
+        break;
+
+
 
 
     ///////////////////////////////////// If liking a post was requested
     case LikePostRequest:
         emit currentJobChanged(tr("Message liked or unliked successfully."));
+
         emit likeSet();
         break;
 
@@ -1331,7 +1477,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
     ///////////////////////////////////// If commenting on a post was requested
     case CommentPostRequest:
-        emit currentJobChanged(tr("Comment posted successfully."));
+        prettyLogMessage = tr("Comment posted successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
+
         emit commentPosted(); // This will be caught by Commenter()
 
         break;
@@ -1362,7 +1511,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     case SharePostRequest:
-        emit currentJobChanged(tr("Post shared successfully."));
+        prettyLogMessage = tr("Post shared successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
+
         break;
 
     case PostSharesRequest:
@@ -1394,23 +1546,39 @@ void PumpController::requestFinished(QNetworkReply *reply)
 
 
     case DeletePostRequest:
-        emit currentJobChanged(tr("Message deleted successfully."));
+        prettyLogMessage = tr("Message deleted successfully.");
+        emit currentJobChanged(prettyLogMessage);
+        emit logMessage(prettyLogMessage);
+
         break;
 
 
     case FollowContactRequest:
-        emit currentJobChanged(tr("Following successfully."));
-
-        // Re-request the contact list, now with a new contact (not very optimized...)
-        this->getContactList("following");
-
-        break;
-
+        // Just jump to next
     case UnfollowContactRequest:
-        emit currentJobChanged(tr("Stopped following successfully."));
+        if (jsonParsedOK && jsonData.size() > 0)
+        {
+            ASPerson *contact = new ASPerson(jsonData.value("object").toMap(),
+                                             this);
 
-        // Re-request the contact list, now with one contact less (not very optimized...)
-        this->getContactList("following");
+            if (requestType == FollowContactRequest)
+            {
+                prettyLogMessage = tr("Following %1 successfully.",
+                                      "%1 is a person's ID").arg(contact->getId());
+
+                emit contactFollowed(contact);
+            }
+            else
+            {
+                prettyLogMessage = tr("Stopped following %1 successfully.",
+                                      "%1 is a person's ID").arg(contact->getId());
+
+                emit contactUnfollowed(contact);
+            }
+
+            emit currentJobChanged(prettyLogMessage);
+            emit logMessage(prettyLogMessage);
+        }
 
         break;
 
@@ -1566,10 +1734,13 @@ void PumpController::requestFinished(QNetworkReply *reply)
         {
             QString personId = jsonData.value("object").toMap().value("id").toString();
             QString personName = jsonData.value("object").toMap().value("displayName").toString();
-            //<< jsonData.value("target").toMap().value("id").toString()
+            QString personAvatar = jsonData.value("object").toMap().value("image").toMap().value("url").toString();
 
-            emit personAddedToList(personId, personName);
-            emit currentJobChanged(tr("Person added to list successfully."));
+            emit personAddedToList(personId, personName, personAvatar);
+
+            prettyLogMessage = tr("Person added to list successfully.");
+            emit currentJobChanged(prettyLogMessage);
+            emit logMessage(prettyLogMessage);
         }
         break;
 
@@ -1581,7 +1752,10 @@ void PumpController::requestFinished(QNetworkReply *reply)
             QString personId = jsonData.value("object").toMap().value("id").toString();
 
             emit personRemovedFromList(personId);
-            emit currentJobChanged(tr("Person removed from list successfully."));
+
+            prettyLogMessage = tr("Person removed from list successfully.");
+            emit currentJobChanged(prettyLogMessage);
+            emit logMessage(prettyLogMessage);
         }
         break;
 
@@ -1618,7 +1792,7 @@ void PumpController::requestFinished(QNetworkReply *reply)
     //////////////////////////////////////// If uploading a file was requested
     case UploadAvatarRequest:
         // just jump to next
-    case UploadPictureRequest:
+    case UploadMediaForPostRequest:
         // just jump
     case UploadFileRequest:
         qDebug() << "Uploading a file was requested";
@@ -1631,12 +1805,15 @@ void PumpController::requestFinished(QNetworkReply *reply)
             qDebug() << "Uploaded file ID:" << uploadedFileID;
 
 
-            if (jsonData["objectType"].toString() == "image")
+            QString objectType = jsonData["objectType"].toString();
+            if (objectType == "image"
+             || objectType == "audio"
+             || objectType == "video")
             {
-                if (requestType == UploadPictureRequest)
+                if (requestType == UploadMediaForPostRequest)
                 {
                     emit currentJobChanged(tr("File uploaded successfully. Posting message..."));
-                    this->postImageStepTwo(uploadedFileID);
+                    this->postMediaStepTwo(uploadedFileID);
                 }
                 else if (requestType == UploadAvatarRequest)
                 {
@@ -1674,7 +1851,7 @@ void PumpController::sslErrorsHandler(QNetworkReply *reply, QList<QSslError> err
 
     // ignore completely for now
     qDebug() << "Ignoring these errors...";
-    reply->ignoreSslErrors();
+    reply->ignoreSslErrors(errorList);
 
     /*
     // Check list of SSL errors
@@ -1689,9 +1866,6 @@ void PumpController::sslErrorsHandler(QNetworkReply *reply, QList<QSslError> err
         }
     }
     */
-
-    // Clean up
-    reply->deleteLater();
 }
 
 
@@ -1760,7 +1934,8 @@ void PumpController::getToken()
 
 
         QString requestTokenURL = "https://" + this->serverURL + "/oauth/request_token";
-        qDebug() << "GET: " << requestTokenURL << "with" << qoauth->consumerKey() << qoauth->consumerSecret();
+        qDebug() << "GET: " << requestTokenURL
+                 << "with" << qoauth->consumerKey() << qoauth->consumerSecret();
 
 
         QOAuth::ParamMap oAuthParams;
@@ -1986,37 +2161,41 @@ void PumpController::postNote(QMap<QString, QString> audienceMap,
 
 
 /*
- * Send an IMAGE to the server
+ * Post media-type of object (image, audio/ video)
  *
  * First, upload the file.
  * Then we get it's ID in a signal, and create the post itself
  *
  */
-void PumpController::postImage(QMap<QString, QString> audienceMap,
-                               QString postText, QString imageTitle,
-                               QString imageFilename, QString contentType)
+void PumpController::postMedia(QMap<QString, QString> audienceMap,
+                               QString postText, QString postTitle,
+                               QString mediaFilename,
+                               QString mediaType,
+                               QString mimeContentType)
 {
-    qDebug() << "PumpController::postImage()";
-    qDebug() << "Uploading" << imageFilename << "with title:" << imageTitle;
+    qDebug() << "PumpController::postMedia()";
+    qDebug() << "Uploading" << mediaFilename << "with title:" << postTitle;
 
-    // Store postText, imageTitle, and audienceMap, then upload
-    this->currentImageTitle = imageTitle;
-    this->currentImageDescription = postText;
+    // Store postTitle, postText, and audienceMap, then upload
+    this->currentPostTitle = postTitle;
+    this->currentPostDescription = postText;
     this->currentAudienceMap = audienceMap;
+    this->currentPostType = mediaType;
 
 
-    this->uploadFile(imageFilename, contentType, UploadPictureRequest);
+    this->uploadFile(mediaFilename, mimeContentType, UploadMediaForPostRequest);
 }
 
 
 /*
- * Post Image, step 2: after getting the ID in the file upload request,
+ * Post Media, step 2: after getting the ID in the file upload request,
  * create the post itself
  *
  */
-void PumpController::postImageStepTwo(QString id)
+void PumpController::postMediaStepTwo(QString id)
 {
-    qDebug() << "PumpController::postImageStepTwo() image ID:" << id;
+    qDebug() << "PumpController::postMediaStepTwo()"
+             <<  currentPostType << "ID:" << id;
 
     QString url = "https://" + this->serverURL + "/api/user/" + this->userName + "/feed";
 
@@ -2024,23 +2203,22 @@ void PumpController::postImageStepTwo(QString id)
 
 
     QVariantMap jsonVariantObject;
-    jsonVariantObject.insert("objectType", "image");
+    jsonVariantObject.insert("objectType", this->currentPostType);
     jsonVariantObject.insert("id", id);
 
+    QList<QVariantList> audience = processAudience(this->currentAudienceMap);
+
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "post");
     jsonVariant.insert("object", jsonVariantObject);
-
-    QList<QVariantList> audience = processAudience(this->currentAudienceMap);
     jsonVariant.insert("to", audience.at(0));
     jsonVariant.insert("cc", audience.at(1));
 
+
     QJson::Serializer serializer;
     QByteArray data = serializer.serialize(jsonVariant);
 
-
     qDebug() << "About to POST:" << data;
-
     nam.post(postRequest, data);
 }
 
@@ -2048,23 +2226,23 @@ void PumpController::postImageStepTwo(QString id)
 /*
  * Workaround for the non-title-non-description issue
  *
- * Update the image post with the title and the description
+ * Update the image/audio/video post with the title and the description
  *
  */
-void PumpController::postImageStepThree(QString id)
+void PumpController::postMediaStepThree(QString id)
 {
-    qDebug() << "PumpController::postImageStepThree() post ID:" << id;
+    qDebug() << "PumpController::postMediaStepThree() post ID:" << id;
 
     // Using the ID directly as URL
     QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
 
     QVariantMap jsonVariant;
     jsonVariant.insert("verb", "update"); // "post" would suffice
-    if (!this->currentImageTitle.isEmpty())
+    if (!this->currentPostTitle.isEmpty())
     {
-        jsonVariant.insert("displayName", this->currentImageTitle);
+        jsonVariant.insert("displayName", this->currentPostTitle);
     }
-    jsonVariant.insert("content", this->currentImageDescription);
+    jsonVariant.insert("content", this->currentPostDescription);
 
     QJson::Serializer serializer;
     QByteArray data = serializer.serialize(jsonVariant);
@@ -2118,7 +2296,7 @@ void PumpController::postAvatarStepTwo(QString id)
 
 
 /*
- * Update a post contents (the object)
+ * Update post contents (the object)
  *
  */
 void PumpController::updatePost(QString id,
@@ -2131,7 +2309,7 @@ void PumpController::updatePost(QString id,
     QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT, UpdatePostRequest);
 
     QVariantMap jsonVariant;
-    jsonVariant.insert("verb", "update"); // "post" would suffice
+    jsonVariant.insert("verb", "update");     // "post" would suffice
     jsonVariant.insert("displayName", title); // "title" can be empty, as a way to remove titles
     jsonVariant.insert("content", content);
 
@@ -2207,6 +2385,30 @@ void PumpController::addComment(QString comment, QString postID, QString postTyp
 }
 
 
+/*
+ * Update comment contents (object)
+ *
+ */
+void PumpController::updateComment(QString id, QString content)
+{
+    qDebug() << "PumpController::updateComment(), comment ID:" << id;
+
+    // Using the ID directly as URL
+    QNetworkRequest postRequest = this->prepareRequest(id, QOAuth::PUT,
+                                                       UpdateCommentRequest);
+
+    QVariantMap jsonVariant;
+    jsonVariant.insert("verb", "update");
+    jsonVariant.insert("content", content);
+
+    QJson::Serializer serializer;
+    QByteArray data = serializer.serialize(jsonVariant);
+
+    qDebug() << "About to PUT:" << data;
+    nam.put(postRequest, data);
+}
+
+
 
 
 void PumpController::sharePost(QString postID, QString postType)
